@@ -22,15 +22,23 @@
 /* Define to 'u_int32_t' if not defined anywhere. */
 #undef __u32
 
-/* Define to 'int32_t' if not defined anywhere. */
+/* Define to 'int64_t' if not defined anywhere. */
 #undef __s64
 
-/* Define to 'u_int32_t' if not defined anywhere. */
+/* Define to 'u_int64_t' if not defined anywhere. */
 #undef __u64
 
 /* Define to 'unsigned' if <sys/types.h> doesn't define. */
 #undef loff_t
 
+#undef DEBUG
+
+#undef READ_ONLY
+
+#undef DISCOVER_ONLY
+
+#undef ENABLE_FS
+
 #undef ENABLE_NLS
 
 #undef ENABLE_MTRACE
@@ -55,3 +63,4 @@
  */
 #undef USE_BLKSSZGET
 
+#undef DYNAMIC_LOADING
