@@ -20,11 +20,14 @@
 #ifndef PED_DEBUG_H_INCLUDED
 #define PED_DEBUG_H_INCLUDED
 
+#include "config.h"
+
 #ifdef DEBUG
 
 #ifdef __GNUC__
 
 #define PED_ASSERT(cond, action)				\
+	do {							\
 	if (!ped_assert (cond,					\
 			 #cond,					\
 			 __FILE__,				\
@@ -32,11 +35,13 @@
 			 __PRETTY_FUNCTION__))			\
 	{							\
 		action;						\
-	}
+	}							\
+	} while (0);
 
 #else /* !__GNUC__ */
 
 #define PED_ASSERT(cond, action)				\
+	do {							\
 	if (!ped_assert (cond,					\
 			 #cond,					\
 			 "unknown",				\
@@ -44,14 +49,15 @@
 			 "unknown"))				\
 	{							\
 		action;						\
-	}
+	}							\
+	} while (0);
 
 
 #endif /* __GNUC__ */
 
 #else /* !DEBUG */
 
-#define PED_ASSERT(cond, action)
+#define PED_ASSERT(cond, action)	while (0) {}
 
 #endif /* DEBUG */
 
