@@ -1,6 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
+    Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,6 +20,8 @@
 #ifndef PED_DEVICE_H_INCLUDED
 #define PED_DEVICE_H_INCLUDED
 
+#include <parted/timer.h>
+
 #define PED_SECTOR_SIZE		512
 typedef long long PedSector;
 
@@ -35,6 +37,7 @@ typedef enum {
 } PedDeviceType;
 
 typedef struct _PedDevice PedDevice;
+typedef struct _PedDeviceArchOps PedDeviceArchOps;
 
 /* A hard disk device - eg. /dev/hda, not /dev/hda3 */
 struct _PedDevice {
@@ -52,14 +55,31 @@ struct _PedDevice {
 	int		external_mode;
 	int		dirty;
 	int		boot_dirty;
-	int		fd;
 
-/* x86-specific: TODO hide this in arch_specific.  But argghh!  The disk
- * label stuff needs it, not to mention front-ends :-( */
 	int		heads, sectors, cylinders;
 	int		geom_known;
 	int		geom_already_guessed;
 	short		host, did;
+
+	void*		arch_specific;
+};
+
+struct _PedDeviceArchOps {
+	PedDevice* (*_new) (const char* path);
+	void (*destroy) (PedDevice* dev);
+	int (*is_busy) (PedDevice* dev);
+	int (*open) (PedDevice* dev);
+	int (*refresh_open) (PedDevice* dev);
+	int (*close) (PedDevice* dev);
+	int (*refresh_close) (PedDevice* dev);
+	int (*read) (PedDevice* dev, void* buffer,
+		     PedSector start, PedSector count);
+	int (*write) (PedDevice* dev, const void* buffer,
+		      PedSector start, PedSector count);
+	int (*sync) (PedDevice* dev);
+	PedSector (*check) (PedDevice* dev, void* buffer,
+			    PedSector start, PedSector count);
+	void (*probe_all) ();
 };
 
 extern void ped_device_probe_all ();
@@ -67,13 +87,15 @@ extern void ped_device_free_all ();
 
 extern PedDevice* ped_device_get (const char* name);
 extern PedDevice* ped_device_get_next (const PedDevice* dev);
+extern int ped_device_is_busy (PedDevice* dev);
 extern int ped_device_open (PedDevice* dev);
 extern int ped_device_close (PedDevice* dev);
+extern void ped_device_destroy (PedDevice* dev);
 
 extern int ped_device_begin_external_access (PedDevice* dev);
 extern int ped_device_end_external_access (PedDevice* dev);
 
-extern int ped_device_read (const PedDevice* dev, void* buffer,
+extern int ped_device_read (PedDevice* dev, void* buffer,
 			    PedSector start, PedSector count);
 extern int ped_device_write (PedDevice* dev, const void* buffer,
 			     PedSector start, PedSector count);
@@ -81,8 +103,9 @@ extern int ped_device_sync (PedDevice* dev);
 extern PedSector ped_device_check (PedDevice* dev, void* buffer,
 				   PedSector start, PedSector count);
 
-/* FIXME */
-extern char* ped_device_get_part_path (PedDevice* dev, int num);
+/* private stuff ;-) */
+
+extern void _ped_device_probe (const char* path);
 
 #endif /* PED_DEVICE_H_INCLUDED */
 
