@@ -1,6 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
+    Copyright (C) 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,30 +20,24 @@
 #ifndef PED_DISK_H_INCLUDED
 #define PED_DISK_H_INCLUDED
 
-typedef struct _PedGeometry		PedGeometry;
 typedef struct _PedDisk			PedDisk;
 typedef struct _PedPartition		PedPartition;
 typedef const struct _PedDiskOps	PedDiskOps;
 typedef struct _PedDiskType		PedDiskType;
+typedef const struct _PedDiskArchOps	PedDiskArchOps;
 
 #include <parted/device.h>
 #include <parted/filesys.h>
 #include <parted/natmath.h>
-
-struct _PedGeometry {
-	PedDisk*		disk;
-	PedSector		start;
-	PedSector		length;
-	PedSector		end;
-};
+#include <parted/geom.h>
 
 typedef enum {
-	PED_PARTITION_PRIMARY		= 0x00,
 	PED_PARTITION_LOGICAL		= 0x01,
 	PED_PARTITION_EXTENDED		= 0x02,
 	PED_PARTITION_FREESPACE		= 0x04,
 	PED_PARTITION_METADATA		= 0x08
 } PedPartitionType;
+/* #define PED_PARTITION_PRIMARY	0 */
 
 typedef enum {
 	PED_PARTITION_BOOT=1,
@@ -52,22 +46,24 @@ typedef enum {
 	PED_PARTITION_HIDDEN=4,
 	PED_PARTITION_RAID=5,
 	PED_PARTITION_LVM=6,
-	PED_PARTITION_LBA=7
+	PED_PARTITION_LBA=7,
+	PED_PARTITION_HPSERVICE=8
 } PedPartitionFlag;
 #define PED_PARTITION_FIRST_FLAG	PED_PARTITION_BOOT
-#define PED_PARTITION_LAST_FLAG		PED_PARTITION_LBA
+#define PED_PARTITION_LAST_FLAG		PED_PARTITION_HPSERVICE
 
 typedef enum {
-	PED_DISK_TYPE_EXTENDED=1,
-	PED_DISK_TYPE_PARTITION_NAME=2
+	PED_DISK_TYPE_EXTENDED=1,	/* supports extended partitions */
+	PED_DISK_TYPE_PARTITION_NAME=2	/* supports partition names */
 } PedDiskTypeFeature;
-#define PED_DISK_TYPE_FIRST_FEATURE	PED_DISK_TYPE_EXTENDED
-#define PED_DISK_TYPE_LAST_FEATURE	PED_DISK_TYPE_PARTITION_NAME
+#define PED_DISK_TYPE_FIRST_FEATURE    PED_DISK_TYPE_EXTENDED
+#define PED_DISK_TYPE_LAST_FEATURE     PED_DISK_TYPE_PARTITION_NAME
 
 struct _PedPartition {
 	PedPartition*		prev;
 	PedPartition*		next;
 
+	PedDisk*		disk;
 	PedGeometry		geom;
 	int			num;
 
@@ -80,22 +76,25 @@ struct _PedPartition {
 
 struct _PedDisk {
 	PedDevice*		dev;
-	PedDiskType*		type;
+	const PedDiskType*	type;
 	PedPartition*		part_list;
 
 	void*			disk_specific;
 
 /* office use only ;-) */
-	int			update_mode;
+	int			needs_clobber;	/* clobber before write? */
+	int			update_mode;	/* mode without free/metadata
+						   partitions, for easier
+						   update */
 };
 
 struct _PedDiskOps {
 	/* disk label operations */
-	int (*probe) (const PedDevice *dev);
-	PedDisk* (*open) (PedDevice* dev);
-	PedDisk* (*create) (PedDevice* dev);
+	int (*probe) (PedDevice *dev);
 	int (*clobber) (PedDevice* dev);
-	int (*close) (PedDisk* disk);
+	PedDisk* (*alloc) (PedDevice* dev);
+	PedDisk* (*duplicate) (const PedDisk* disk);
+	void (*free) (PedDisk* disk);
 	int (*read) (PedDisk* disk);
 	int (*write) (PedDisk* disk);
 
@@ -106,7 +105,10 @@ struct _PedDiskOps {
 		const PedFileSystemType* fs_type,
 		PedSector start,
 		PedSector end);
+	PedPartition* (*partition_duplicate) (const PedPartition* part);
 	void (*partition_destroy) (PedPartition* part);
+	int (*partition_set_system) (PedPartition* part,
+				     const PedFileSystemType* fs_type);
 	int (*partition_set_flag) (
 		PedPartition* part,
 	       	PedPartitionFlag flag,
@@ -122,7 +124,6 @@ struct _PedDiskOps {
 	int (*partition_align) (PedPartition* part,
 				const PedConstraint* constraint);
 	int (*partition_enumerate) (PedPartition* part);
-	int (*partition_set_extended_system) (PedPartition* part);
 
 	/* other */
 	int (*alloc_metadata) (PedDisk* disk);
@@ -134,71 +135,44 @@ struct _PedDiskType {
 	const char*		name;
 	PedDiskOps* const	ops;
 
-	PedDiskTypeFeature	features;
+	PedDiskTypeFeature	features;   /* bitmap of supported features */
+};
+
+/* Architecture specific operations.  i.e. communication with kernel (or
+ * whatever) about changes, etc.
+ */
+struct _PedDiskArchOps {
+	char* (*partition_get_path) (const PedPartition* part);
+	int (*partition_is_busy) (const PedPartition* part);
+	int (*disk_commit) (PedDisk* disk);
 };
 
 extern void ped_register_disk_type (PedDiskType* type);
 extern void ped_unregister_disk_type (PedDiskType* type);
 extern PedDiskType* ped_disk_type_get_next (PedDiskType* type);
 extern PedDiskType* ped_disk_type_get (const char* name);
-extern PedDiskType* ped_disk_probe (const PedDevice* dev);
 extern int ped_disk_type_check_feature (const PedDiskType* disk_type,
 	       				PedDiskTypeFeature feature);
 
-extern PedDisk* ped_disk_open (PedDevice* dev);
-extern PedDisk* ped_disk_create (PedDevice* dev, PedDiskType* type);
-extern int ped_disk_clobber (PedDisk* disk);
-extern int ped_disk_close (PedDisk* disk);
-extern int ped_disk_read (PedDisk* disk);
-extern int ped_disk_write (PedDisk* disk);
-extern int ped_disk_is_busy (const PedDisk* disk);
+extern PedDiskType* ped_disk_probe (PedDevice* dev);
+extern int ped_disk_clobber (PedDevice* dev);
+extern int ped_disk_clobber_exclude (PedDevice* dev,
+				     const PedDiskType* exclude);
+extern PedDisk* ped_disk_new (PedDevice* dev);
+extern PedDisk* ped_disk_new_fresh (PedDevice* dev,
+				    const PedDiskType* disk_type);
+extern PedDisk* ped_disk_duplicate (const PedDisk* old_disk);
+extern void ped_disk_destroy (PedDisk* disk);
+extern int ped_disk_commit (PedDisk* disk);
+extern int ped_disk_commit_to_dev (PedDisk* disk);
+extern int ped_disk_commit_to_os (PedDisk* disk);
+extern int ped_disk_check (PedDisk* disk);
+extern void ped_disk_print (PedDisk* disk);
 
-int ped_disk_get_primary_partition_count (PedDisk* disk);
-int ped_disk_get_last_partition_num (PedDisk* disk);
+extern int ped_disk_get_primary_partition_count (PedDisk* disk);
+extern int ped_disk_get_last_partition_num (PedDisk* disk);
 extern int ped_disk_get_max_primary_partition_count (const PedDisk* disk);
 
-extern PedDisk* ped_disk_alloc (PedDevice* dev, PedDiskType* type);
-extern void ped_disk_free (PedDisk* disk);
-
-extern int ped_geometry_init (PedGeometry* geom, const PedDisk* disk,
-			      PedSector start, PedSector length);
-extern PedGeometry* ped_geometry_new (const PedDisk* disk, PedSector start,
-				      PedSector length);
-extern PedGeometry* ped_geometry_duplicate (const PedGeometry* geom);
-extern PedGeometry* ped_geometry_intersect (const PedGeometry* a,
-	       				    const PedGeometry* b);
-extern void ped_geometry_destroy (PedGeometry* geom);
-extern int ped_geometry_set (PedGeometry* geom, PedSector start,
-			     PedSector length);
-extern int ped_geometry_set_start (PedGeometry* geom, PedSector start);
-extern int ped_geometry_set_end (PedGeometry* geom, PedSector end);
-extern int ped_geometry_test_overlap (const PedGeometry* a,
-	       			      const PedGeometry* b);
-extern int ped_geometry_test_inside (const PedGeometry* a,
-				     const PedGeometry* b);
-extern int ped_geometry_test_equal (const PedGeometry* a, const PedGeometry* b);
-extern int ped_geometry_test_sector_inside (const PedGeometry* geom,
-	       				    PedSector sect);
-
-extern int ped_geometry_read (const PedGeometry* geom, void* buffer,
-			      PedSector offset, PedSector count);
-extern int ped_geometry_write (PedGeometry* geom, const void* buffer,
-			       PedSector offset, PedSector count);
-extern PedSector ped_geometry_check (PedGeometry* geom, void* buffer,
-				     PedSector buffer_size, PedSector offset,
-				     PedSector granularity, PedSector count);
-extern int ped_geometry_sync (PedGeometry* geom);
-
-/* returns -1 if "sector" is not within dest's space. */
-extern PedSector ped_geometry_map (const PedGeometry* dst,
-				   const PedGeometry* src,
-				   PedSector sector);
-
-extern PedPartition* ped_partition_alloc (const PedDisk* disk,
-					  PedPartitionType type,
-					  const PedFileSystemType* fs_type,
-					  PedSector start,
-					  PedSector end);
 extern PedPartition* ped_partition_new (const PedDisk* disk,
 					PedPartitionType type,
 					const PedFileSystemType* fs_type,
@@ -217,6 +191,7 @@ extern int ped_partition_set_system (PedPartition* part,
 extern int ped_partition_set_name (PedPartition* part, const char* name);
 extern const char* ped_partition_get_name (const PedPartition* part);
 extern int ped_partition_is_busy (const PedPartition* part);
+extern char* ped_partition_get_path (const PedPartition* part);
 
 extern const char* ped_partition_type_get_name (PedPartitionType part_type);
 extern const char* ped_partition_flag_get_name (PedPartitionFlag flag);
@@ -244,5 +219,20 @@ extern PedPartition* ped_disk_get_partition_by_sector (const PedDisk* disk,
 						       PedSector sect);
 extern PedPartition* ped_disk_extended_partition (const PedDisk* disk);
 
+/* internal functions */
+extern PedDisk* _ped_disk_alloc (PedDevice* dev, const PedDiskType* type);
+extern void _ped_disk_free (PedDisk* disk);
+
+extern PedPartition* _ped_partition_alloc (const PedDisk* disk,
+					   PedPartitionType type,
+					   const PedFileSystemType* fs_type,
+					   PedSector start,
+					   PedSector end);
+extern void _ped_partition_free (PedPartition* part);
+
+extern int _ped_partition_attempt_align (
+		PedPartition* part, const PedConstraint* external,
+		PedConstraint* internal);
+
 #endif /* PED_DISK_H_INCLUDED */
 
