@@ -1,6 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 2000 Free Software Foundation, Inc.
+    Copyright (C) 2000, 2001 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,15 +19,13 @@
 
 #include "config.h"
 
-#ifdef ENABLE_PC98
-
 #include <parted/parted.h>
+#include <parted/debug.h>
 #include <parted/endian.h>
-#include <parted/disk_pc98.h>
 #include <string.h>
 
-#include <libintl.h>
 #if ENABLE_NLS
+#  include <libintl.h>
 #  define _(String) dgettext (PACKAGE, String)
 #else
 #  define _(String) (String)
@@ -85,28 +83,36 @@ typedef struct _PC98RawTable		PC98RawTable;
 
 /* ripped from Linux/98 source */
 struct _PC98RawPartition {
-	__u8	mid;		/* 0x80 - boot */
-	__u8	sid;		/* 0x80 - active */
-	__u8	dum1;		/* dummy for padding */
-	__u8	dum2;		/* dummy for padding */
-	__u8	ipl_sect;	/* IPL sector */
-	__u8	ipl_head;	/* IPL head */
-	__u16	ipl_cyl;	/* IPL cylinder */
-	__u8	sector;		/* starting sector */
-	__u8	head;		/* starting head */
-	__u16	cyl;		/* starting cylinder */
-	__u8	end_sector;	/* end sector */
-	__u8	end_head;	/* end head */
-	__u16	end_cyl;	/* end cylinder */
-	__u8	name[16];
+	uint8_t		mid;		/* 0x80 - boot */
+	uint8_t		sid;		/* 0x80 - active */
+	uint8_t		dum1;		/* dummy for padding */
+	uint8_t		dum2;		/* dummy for padding */
+	uint8_t		ipl_sect;	/* IPL sector */
+	uint8_t		ipl_head;	/* IPL head */
+	uint16_t	ipl_cyl;	/* IPL cylinder */
+	uint8_t		sector;		/* starting sector */
+	uint8_t		head;		/* starting head */
+	uint16_t	cyl;		/* starting cylinder */
+	uint8_t		end_sector;	/* end sector */
+	uint8_t		end_head;	/* end head */
+	uint16_t	end_cyl;	/* end cylinder */
+	uint8_t		name[16];
 } __attribute__((packed));
 
 struct _PC98RawTable {
-	__u8			boot_code [510];
-	__u16			magic;
+	uint8_t			boot_code [510];
+	uint16_t		magic;
 	PC98RawPartition	partitions [MAX_PART_COUNT];
 } __attribute__((packed));
 
+typedef struct {
+	PedSector	ipl_sector;
+	int		system;
+	int		boot;
+	int		hidden;
+	char		name [17];
+} PC98PartitionData;
+
 /* this MBR boot code is dummy */
 static char MBR_BOOT_CODE[] = {
 	0xcb,			/* retf */
@@ -114,87 +120,13 @@ static char MBR_BOOT_CODE[] = {
 	0x49, 0x50, 0x4c, 0x31  /* "IPL1" */
 };
 
-void ped_disk_pc98_init ();
-void ped_disk_pc98_done ();
-
-static int pc98_probe (const PedDevice *dev);
-static PedDisk* pc98_open (PedDevice* dev);
-static PedDisk* pc98_create (PedDevice* dev);
-static int pc98_clobber (PedDevice* dev);
-static int pc98_close (PedDisk* disk);
-static int pc98_read (PedDisk* disk);
-static int pc98_write (PedDisk* disk);
-
-static PedPartition* pc98_partition_new (
-	const PedDisk* disk, PedPartitionType part_type,
-	const PedFileSystemType* fs_type, PedSector start, PedSector end);
-static void pc98_partition_destroy (PedPartition* part);
-static int pc98_partition_set_flag (
-	PedPartition* part, PedPartitionFlag flag, int state);
-static int pc98_partition_get_flag (
-	const PedPartition* part, PedPartitionFlag flag);
-static int pc98_partition_is_flag_available (
-	const PedPartition* part,
-	PedPartitionFlag flag);
-static void pc98_partition_set_name (PedPartition* part, const char* name);
-static const char* pc98_partition_get_name (const PedPartition* part);
-static int pc98_partition_align (PedPartition* part,
-				 const PedConstraint* constraint);
-static int pc98_partition_enumerate (PedPartition* part);
-
-static int pc98_alloc_metadata (PedDisk* disk);
-static int pc98_get_max_primary_partition_count (const PedDisk* disk);
+static PedDiskType pc98_disk_type;
 
 static int is_unused_partition(const PC98RawPartition* raw_part);
 static PedSector chs_to_sector (const PedDevice* dev, int c, int h, int s);
 static void sector_to_chs (const PedDevice* dev, PedSector sector,
 			   int* c, int* h, int* s);
 
-static PedDiskOps pc98_disk_ops = {
-	probe:			pc98_probe,
-	open:			pc98_open,
-	create:			pc98_create,
-	clobber:		pc98_clobber,
-	close:			pc98_close,
-	read:			pc98_read,
-	write:			pc98_write,
-	
-	partition_new:		pc98_partition_new,
-	partition_destroy:	pc98_partition_destroy,
-	partition_set_flag:	pc98_partition_set_flag,
-	partition_get_flag:	pc98_partition_get_flag,
-	partition_is_flag_available:	pc98_partition_is_flag_available,
-	partition_set_name:	pc98_partition_set_name,
-	partition_get_name:	pc98_partition_get_name,
-	partition_align:	pc98_partition_align,
-	partition_enumerate:	pc98_partition_enumerate,
-	partition_set_extended_system:	NULL,
-
-	alloc_metadata:		pc98_alloc_metadata,
-	get_max_primary_partition_count:
-				pc98_get_max_primary_partition_count
-};
-
-static PedDiskType pc98_disk_type = {
-	next:		NULL,
-	name:		"pc98",
-	ops:		&pc98_disk_ops,
-	features:	PED_DISK_TYPE_PARTITION_NAME
-};
-
-void
-ped_disk_pc98_init ()
-{
-	PED_ASSERT (sizeof (PC98RawTable) == 512 * 2, return);
-	ped_register_disk_type (&pc98_disk_type);
-}
-
-void
-ped_disk_pc98_done ()
-{
-	ped_unregister_disk_type (&pc98_disk_type);
-}
-
 /* magic(?) check */
 static int
 pc98_check_magic (const PC98RawTable *part_table)
@@ -241,7 +173,7 @@ check_partition_consistency (const PedDevice* dev,
 }
 
 static int
-pc98_probe (const PedDevice *dev)
+pc98_probe (PedDevice *dev)
 {
 	PC98RawTable		part_table;
 	int			empty;
@@ -249,16 +181,11 @@ pc98_probe (const PedDevice *dev)
 
 	PED_ASSERT (dev != NULL, return 0);
 
-	if (!ped_device_open ((PedDevice*) dev))
+	if (!ped_device_read (dev, &part_table, 0, 2))
 		return 0;
-	if (!ped_device_read (dev, &part_table, 0, 2)) {
-		ped_device_close ((PedDevice*) dev);
-		return 0;
-	}
-	ped_device_close ((PedDevice*) dev);
 
 	/* check magic */
-	if (!pc98_check_magic(&part_table))
+	if (!pc98_check_magic (&part_table))
 		return 0;
 
 	/* check consistency */
@@ -288,75 +215,7 @@ pc98_probe (const PedDevice *dev)
 	return 1;
 }
 
-static PedDisk*
-pc98_open (PedDevice* dev)
-{
-	PedDisk*	disk;
-
-	PED_ASSERT (dev != NULL, return 0);
-
-	if (!pc98_probe (dev))
-		goto error;
-
-	ped_device_open ((PedDevice*) dev);
-	disk = ped_disk_alloc (dev, &pc98_disk_type);
-	if (!disk)
-		goto error;
-
-	if (!pc98_read (disk))
-		goto error_free_disk;
-
-	return disk;
-
-error_free_disk:
-	ped_disk_free (disk);
-error:
-	return NULL;
-}
-
-static PedDisk*
-pc98_create (PedDevice* dev)
-{
-	PC98RawTable		 table;
-
-	PED_ASSERT (dev != NULL, return NULL);
-
-	if (!ped_device_open (dev))
-		goto error;
-	if (!ped_device_read (dev, &table, 0, 2))
-		goto error_close_dev;
-
-	if (!pc98_check_ipl_signature (&table)) {
-		memset (table.boot_code, 0, sizeof (table.boot_code));
-		memcpy (table.boot_code, MBR_BOOT_CODE, sizeof(MBR_BOOT_CODE));
-	}
-
-	memset (table.partitions, 0, sizeof (table.partitions));
-	table.magic = PED_CPU_TO_LE16(PC9800_EXTFMT_MAGIC);
-
-	if (!ped_device_write (dev, (void*) &table, 0, 2))
-		goto error_close_dev;
-	if (!ped_device_sync (dev))
-		goto error_close_dev;
-	ped_device_close (dev);
-	return pc98_open (dev);
-
-error_close_dev:
-	ped_device_close (dev);
-error:
-	return NULL;
-}
-
-static int
-pc98_close (PedDisk* disk)
-{
-	PED_ASSERT (disk != NULL, return 0);
-
-	ped_device_close (disk->dev);
-	ped_disk_free (disk);
-	return 1;
-}
-
+#ifndef DISCOVER_ONLY
 static int
 pc98_clobber (PedDevice* dev)
 {
@@ -365,10 +224,8 @@ pc98_clobber (PedDevice* dev)
 	PED_ASSERT (dev != NULL, return 0);
 	PED_ASSERT (pc98_probe (dev), return 0);
 
-	if (!ped_device_open ((PedDevice*) dev))
-		goto error;
 	if (!ped_device_read (dev, &table, 0, 1))
-		goto error_close_dev;
+		return 0;
 
 	memset (table.partitions, 0, sizeof (table.partitions));
 	table.magic = PED_CPU_TO_LE16(0);
@@ -377,16 +234,33 @@ pc98_clobber (PedDevice* dev)
 		memset (table.boot_code, 0, sizeof (table.boot_code));
 
 	if (!ped_device_write (dev, (void*) &table, 0, 1))
-		goto error_close_dev;
-	if (!ped_device_sync (dev))
-		goto error_close_dev;
-	ped_device_close (dev);
-	return 1;
+		return 0;
+	return ped_device_sync (dev);
+}
+#endif /* !DISCOVER_ONLY */
 
-error_close_dev:
-	ped_device_close (dev);
-error:
-	return 0;
+static PedDisk*
+pc98_alloc (PedDevice* dev)
+{
+	PedDisk*	disk;
+
+	PED_ASSERT (dev != NULL, return 0);
+
+	return _ped_disk_alloc (dev, &pc98_disk_type);
+}
+
+static PedDisk*
+pc98_duplicate (const PedDisk* disk)
+{
+	return ped_disk_new_fresh (disk->dev, &pc98_disk_type);
+}
+
+static void
+pc98_free (PedDisk* disk)
+{
+	PED_ASSERT (disk != NULL, return);
+
+	_ped_disk_free (disk);
 }
 
 static PedSector
@@ -459,11 +333,13 @@ read_table (PedDisk* disk)
 {
 	int			i;
 	PC98RawTable		table;
-	PedConstraint*		constraint_any = ped_constraint_any (disk);
+	PedConstraint*		constraint_any;
 
 	PED_ASSERT (disk != NULL, return 0);
 	PED_ASSERT (disk->dev != NULL, return 0);
 
+	constraint_any = ped_constraint_any (disk->dev);
+
 	if (!ped_device_read (disk->dev, (void*) &table, 0, 2))
 		goto error;
 
@@ -490,8 +366,7 @@ read_table (PedDisk* disk)
 		part_start = legacy_start (disk, raw_part);
 		part_end   = legacy_end (disk, raw_part);
 
-		part = ped_partition_new (disk, PED_PARTITION_PRIMARY, NULL,
-					  part_start, part_end);
+		part = ped_partition_new (disk, 0, NULL, part_start, part_end);
 		if (!part)
 			goto error;
 		pc98_data = part->disk_specific;
@@ -551,6 +426,7 @@ pc98_read (PedDisk* disk)
 	return read_table (disk);
 }
 
+#ifndef DISCOVER_ONLY
 static int
 fill_raw_part (PC98RawPartition* raw_part, const PedPartition* part)
 {
@@ -577,13 +453,13 @@ fill_raw_part (PC98RawPartition* raw_part, const PedPartition* part)
 		name = part->fs_type->name;
 	memcpy (raw_part->name, name, strlen (name));
 
-	sector_to_chs (part->geom.disk->dev, part->geom.start, &c, &h, &s);
+	sector_to_chs (part->disk->dev, part->geom.start, &c, &h, &s);
 	raw_part->cyl	 = PED_CPU_TO_LE16(c);
 	raw_part->head	 = h;
 	raw_part->sector = s;
 
 	if (pc98_data->ipl_sector) {
-		sector_to_chs (part->geom.disk->dev, pc98_data->ipl_sector,
+		sector_to_chs (part->disk->dev, pc98_data->ipl_sector,
 			       &c, &h, &s);
 		raw_part->ipl_cyl  = PED_CPU_TO_LE16(c);
 		raw_part->ipl_head = h;
@@ -594,9 +470,9 @@ fill_raw_part (PC98RawPartition* raw_part, const PedPartition* part)
 		raw_part->ipl_sect = raw_part->sector;
 	}
 
-	sector_to_chs (part->geom.disk->dev, part->geom.end, &c, &h, &s);
-	if (h != part->geom.disk->dev->heads - 1
-	    || s != part->geom.disk->dev->sectors - 1) {
+	sector_to_chs (part->disk->dev, part->geom.end, &c, &h, &s);
+	if (h != part->disk->dev->heads - 1
+	    || s != part->disk->dev->sectors - 1) {
 		ped_exception_throw (
 		    PED_EXCEPTION_NO_FEATURE,
 		    PED_EXCEPTION_CANCEL,
@@ -627,7 +503,8 @@ pc98_write (PedDisk* disk)
 	PED_ASSERT (disk != NULL, return 0);
 	PED_ASSERT (disk->dev != NULL, return 0);
 
-	ped_device_read (disk->dev, &table, 0, 2);
+	if (!ped_device_read (disk->dev, &table, 0, 2))
+		return 0;
 
 	if (!pc98_check_ipl_signature (&table)) {
 		memset (table.boot_code, 0, sizeof(table.boot_code));
@@ -648,10 +525,9 @@ pc98_write (PedDisk* disk)
 
 	if (!ped_device_write (disk->dev, (void*) &table, 0, 2))
 		return 0;
-	if (!ped_device_sync (disk->dev))
-		return 0;
-	return 1;
+	return ped_device_sync (disk->dev);
 }
+#endif /* !DISCOVER_ONLY */
 
 static PedPartition*
 pc98_partition_new (
@@ -661,7 +537,7 @@ pc98_partition_new (
 	PedPartition*		part;
 	PC98PartitionData*	pc98_data;
 
-	part = ped_partition_alloc (disk, part_type, fs_type, start, end);
+	part = _ped_partition_alloc (disk, part_type, fs_type, start, end);
 	if (!part)
 		goto error;
 
@@ -687,6 +563,27 @@ error:
 	return 0;
 }
 
+static PedPartition*
+pc98_partition_duplicate (const PedPartition* part)
+{
+	PedPartition*		new_part;
+	PC98PartitionData*	new_pc98_data;
+	PC98PartitionData*	old_pc98_data;
+
+	new_part = ped_partition_new (part->disk, part->type,
+				      part->fs_type, part->geom.start,
+				      part->geom.end);
+	if (!new_part)
+		return NULL;
+
+	old_pc98_data = (PC98PartitionData*) part->disk_specific;
+	new_pc98_data = (PC98PartitionData*) new_part->disk_specific;
+
+	/* ugly, but C is ugly :p */
+	memcpy (new_pc98_data, old_pc98_data, sizeof (PC98PartitionData));
+	return new_part;
+}
+
 static void
 pc98_partition_destroy (PedPartition* part)
 {
@@ -697,32 +594,59 @@ pc98_partition_destroy (PedPartition* part)
 	ped_free (part);
 }
 
+static int
+pc98_partition_set_system (PedPartition* part, const PedFileSystemType* fs_type)
+{
+	PC98PartitionData* pc98_data = part->disk_specific;
+	
+	part->fs_type = fs_type;
+
+	pc98_data->system = 0x2062;
+	if (fs_type) {
+		if (!strcmp (fs_type->name, "fat16")) {
+			if (part->geom.length * 512 >= 32 * 1024 * 1024)
+				pc98_data->system = 0x2021;
+			else
+				pc98_data->system = 0x2011;
+		} else if (!strcmp (fs_type->name, "fat32")) {
+			pc98_data->system = 0x2061;
+		} else if (!strcmp (fs_type->name, "ntfs")) {
+			pc98_data->system = 0x2031;
+		} else if (!strncmp (fs_type->name, "ufs", 3)) {
+			pc98_data->system = 0x2044;
+		} else { /* ext2, reiser, xfs, etc. */
+			/* ext2 partitions must be marked boot */
+			pc98_data->boot = 1;
+			pc98_data->system = 0xa062;
+		}
+	}
+
+	if (pc98_data->boot)
+		pc98_data->system |= 0x8000;
+	if (!pc98_data->hidden)
+		pc98_data->system |= 0x0080;
+	return 1;
+}
+
 static int
 pc98_partition_set_flag (PedPartition* part, PedPartitionFlag flag, int state)
 {
 	PedDisk*			disk;
 	PC98PartitionData*		pc98_data;
-	const PedFileSystemType*	fs_type;
 
 	PED_ASSERT (part != NULL, return 0);
 	PED_ASSERT (part->disk_specific != NULL, return 0);
-	PED_ASSERT (part->geom.disk != NULL, return 0);
 
 	pc98_data = part->disk_specific;
-	disk = part->geom.disk;
-
-	/* hack! */
-	fs_type = part->fs_type ? part->fs_type
-				: ped_file_system_type_get ("ext2");
 
 	switch (flag) {
 	case PED_PARTITION_HIDDEN:
 		pc98_data->hidden = state;
-		return ped_partition_set_system (part, fs_type);
+		return ped_partition_set_system (part, part->fs_type);
 
 	case PED_PARTITION_BOOT:
 		pc98_data->boot = state;
-		return ped_partition_set_system (part, fs_type);
+		return ped_partition_set_system (part, part->fs_type);
 
 	default:
 		return 0;
@@ -795,50 +719,24 @@ pc98_partition_get_name (const PedPartition* part)
 static PedConstraint*
 _primary_constraint (PedDisk* disk)
 {
+	PedDevice*	dev = disk->dev;
 	PedAlignment	start_align;
 	PedAlignment	end_align;
 	PedGeometry	max_geom;
 	PedSector	cylinder_size;
 
-	cylinder_size = disk->dev->sectors * disk->dev->heads;
+	cylinder_size = dev->sectors * dev->heads;
 
 	if (!ped_alignment_init (&start_align, 0, cylinder_size))
 		return NULL;
 	if (!ped_alignment_init (&end_align, -1, cylinder_size))
 		return NULL;
-	if (!ped_geometry_init (&max_geom, disk, cylinder_size,
-			       	disk->dev->length - cylinder_size))
+	if (!ped_geometry_init (&max_geom, dev, cylinder_size,
+			       	dev->length - cylinder_size))
 		return NULL;
 
 	return ped_constraint_new (&start_align, &end_align, &max_geom,
-				   &max_geom, 1);
-}
-
-/* argghh!  this should be somewhere else... but where?! */
-static int
-_try_constraint (PedPartition* part, const PedConstraint* external,
-		 PedConstraint* internal)
-{
-	PedConstraint*		intersection;
-	PedGeometry*		solution;
-
-	intersection = ped_constraint_intersect (external, internal);
-	ped_constraint_destroy (internal);
-	if (!intersection)
-		goto fail;
-
-	solution = ped_constraint_solve_nearest (intersection, &part->geom);
-	if (!solution)
-		goto fail_free_intersection;
-	ped_geometry_set (&part->geom, solution->start, solution->length);
-	ped_geometry_destroy (solution);
-	ped_constraint_destroy (intersection);
-	return 1;
-
-fail_free_intersection:
-	ped_constraint_destroy (intersection);
-fail:
-	return 0;
+				   &max_geom, 1, dev->length);
 }
 
 static int
@@ -846,15 +744,16 @@ pc98_partition_align (PedPartition* part, const PedConstraint* constraint)
 {
 	PED_ASSERT (part != NULL, return 0);
 
-	if (_try_constraint (part, constraint,
-			     _primary_constraint (part->geom.disk)))
-		return 1;
+	if (_ped_partition_attempt_align (part, constraint,
+					  _primary_constraint (part->disk)))
+	       	return 1;
 
+#ifndef DISCOVER_ONLY
 	ped_exception_throw (
 		PED_EXCEPTION_ERROR,
 		PED_EXCEPTION_CANCEL,
-		_("Unable to align partition."));
-
+		_("Unable to satisfy all constraints on the partition."));
+#endif
 	return 0;
 }
 
@@ -872,20 +771,16 @@ next_primary (PedDisk* disk)
 static int
 pc98_partition_enumerate (PedPartition* part)
 {
-	PedDisk*	disk;
-
 	PED_ASSERT (part != NULL, return 0);
-	PED_ASSERT (part->geom.disk != NULL, return 0);
-
-	disk = part->geom.disk;
+	PED_ASSERT (part->disk != NULL, return 0);
 
 	/* don't re-number a partition */
 	if (part->num != -1)
 		return 1;
 
-	PED_ASSERT (part->type == PED_PARTITION_PRIMARY, return 0);
+	PED_ASSERT (ped_partition_is_active (part), return 0);
 
-	part->num = next_primary (disk);
+	part->num = next_primary (part->disk);
 	if (!part->num) {
 		ped_exception_throw (PED_EXCEPTION_ERROR,
 			PED_EXCEPTION_CANCEL,
@@ -900,16 +795,15 @@ static int
 pc98_alloc_metadata (PedDisk* disk)
 {
 	PedPartition*		new_part;
-	PedConstraint*		constraint_any = ped_constraint_any (disk);
+	PedConstraint*		constraint_any = NULL;
 
 	PED_ASSERT (disk != NULL, goto error);
 	PED_ASSERT (disk->dev != NULL, goto error);
 
-	new_part = ped_partition_new (
-			disk,
-			PED_PARTITION_PRIMARY | PED_PARTITION_METADATA,
-			NULL,
-			0, disk->dev->sectors * disk->dev->heads - 1);
+	constraint_any = ped_constraint_any (disk->dev);
+
+	new_part = ped_partition_new (disk, PED_PARTITION_METADATA, NULL,
+				0, disk->dev->sectors * disk->dev->heads - 1);
 	if (!new_part)
 		goto error;
 
@@ -932,4 +826,56 @@ pc98_get_max_primary_partition_count (const PedDisk* disk)
 	return MAX_PART_COUNT;
 }
 
-#endif /* ENABLE_PC98 */
+static PedDiskOps pc98_disk_ops = {
+	probe:			pc98_probe,
+#ifndef DISCOVER_ONLY
+	clobber:		pc98_clobber,
+#else
+	clobber:		NULL,
+#endif
+	alloc:			pc98_alloc,
+	duplicate:		pc98_duplicate,
+	free:			pc98_free,
+	read:			pc98_read,
+#ifndef DISCOVER_ONLY
+	write:			pc98_write,
+#else
+	write:			NULL,
+#endif
+	
+	partition_new:		pc98_partition_new,
+	partition_duplicate:	pc98_partition_duplicate,
+	partition_destroy:	pc98_partition_destroy,
+	partition_set_system:	pc98_partition_set_system,
+	partition_set_flag:	pc98_partition_set_flag,
+	partition_get_flag:	pc98_partition_get_flag,
+	partition_is_flag_available:	pc98_partition_is_flag_available,
+	partition_set_name:	pc98_partition_set_name,
+	partition_get_name:	pc98_partition_get_name,
+	partition_align:	pc98_partition_align,
+	partition_enumerate:	pc98_partition_enumerate,
+
+	alloc_metadata:		pc98_alloc_metadata,
+	get_max_primary_partition_count:
+				pc98_get_max_primary_partition_count
+};
+
+static PedDiskType pc98_disk_type = {
+	next:		NULL,
+	name:		"pc98",
+	ops:		&pc98_disk_ops,
+	features:	PED_DISK_TYPE_PARTITION_NAME
+};
+
+void
+ped_disk_pc98_init ()
+{
+	PED_ASSERT (sizeof (PC98RawTable) == 512 * 2, return);
+	ped_register_disk_type (&pc98_disk_type);
+}
+
+void
+ped_disk_pc98_done ()
+{
+	ped_unregister_disk_type (&pc98_disk_type);
+}
