@@ -20,6 +20,7 @@
 #include "config.h"
 
 #include <parted/parted.h>
+#include <parted/debug.h>
 #include <string.h>
 
 #include <libintl.h>
@@ -88,15 +89,32 @@ ped_file_system_type_get_next (const PedFileSystemType* fs_type)
 		return fs_types;
 }
 
+PedGeometry*
+ped_file_system_probe_specific (
+		const PedFileSystemType* fs_type, PedGeometry* geom)
+{
+	PedGeometry*	result;
+
+	PED_ASSERT (fs_type != NULL, return NULL);
+	PED_ASSERT (fs_type->ops->probe != NULL, return NULL);
+	PED_ASSERT (geom != NULL, return NULL);
+
+	if (!ped_device_open (geom->dev))
+		return 0;
+	result = fs_type->ops->probe (geom);
+	ped_device_close (geom->dev);
+	return result;
+}
+
 static int
-_test_open (PedFileSystemType* fs_type, const PedGeometry* geom)
+_test_open (PedFileSystemType* fs_type, PedGeometry* geom)
 {
 	PedFileSystem*		fs;
 
 	ped_exception_fetch_all ();
 	fs = fs_type->ops->open (geom);
 	if (fs)
-		ped_file_system_close (fs);
+		fs_type->ops->close (fs);
 	else
 		ped_exception_catch ();
 	ped_exception_leave_all ();
@@ -104,46 +122,106 @@ _test_open (PedFileSystemType* fs_type, const PedGeometry* geom)
 }
 
 static PedFileSystemType*
-_probe_with_open (const PedGeometry* geom, int detected_count,
+_probe_with_open (PedGeometry* geom, int detected_count,
 		  PedFileSystemType* detected[])
 {
 	int			i;
 	PedFileSystemType*	open_detected = NULL;
 
+	ped_device_open (geom->dev);
+
 	for (i=0; i<detected_count; i++) {
 		if (detected [i]->ops->open && !_test_open (detected [i], geom))
 			continue;
 
-		if (open_detected)
+		if (open_detected) {
+			ped_device_close (geom->dev);
 			return NULL;
-		else
+		} else {
 			open_detected = detected [i];
+		}
 	}
+
+	ped_device_close (geom->dev);
 	return open_detected;
 }
 
+static int
+_geometry_error (const PedGeometry* a, const PedGeometry* b)
+{
+	PedSector	start_delta = a->start - b->start;
+	PedSector	end_delta = a->end - b->end;
+
+	return abs (start_delta) + abs (end_delta);
+}
+
+static PedFileSystemType*
+_best_match (const PedGeometry* geom, PedFileSystemType* detected [],
+	     const int detected_error [], int detected_count)
+{
+	int		best_match = 0;
+	int		i;
+	PedSector	min_error;
+
+	min_error = PED_MAX (4096, geom->length / 100);
+
+	for (i = 1; i < detected_count; i++) {
+		if (detected_error [i] < detected_error [best_match])
+			best_match = i;
+	}
+
+	/* make sure the best match is significantly better than all the
+	 * other matches
+	 */
+	for (i = 0; i < detected_count; i++) {
+		if (i == best_match)
+			continue;
+
+		if (abs (detected_error [best_match] - detected_error [i])
+				< min_error)
+			return NULL;
+	}
+
+	return detected [best_match];
+}
+
 PedFileSystemType*
-ped_file_system_probe (const PedGeometry* geom)
+ped_file_system_probe (PedGeometry* geom)
 {
 	PedFileSystemType*	detected[32];
+	int			detected_error[32];
 	int			detected_count = 0;
 	PedFileSystemType*	walk = NULL;
 
 	PED_ASSERT (geom != NULL, return NULL);
 
+	if (!ped_device_open (geom->dev))
+		return NULL;
+
 	ped_exception_fetch_all ();
 	while ( (walk = ped_file_system_type_get_next (walk)) ) {
-		if (walk->ops->probe (geom))
-			detected [detected_count++] = walk;
-		else
+		PedGeometry*	probed;
+
+		probed = ped_file_system_probe_specific (walk, geom);
+		if (probed) {
+			detected [detected_count] = walk;
+			detected_error [detected_count]
+				= _geometry_error (geom, probed);
+			detected_count++;
+			ped_geometry_destroy (probed);
+		} else {
 			ped_exception_catch ();
+		}
 	}
 	ped_exception_leave_all ();
 
+	ped_device_close (geom->dev);
+
 	if (!detected_count)
 		return NULL;
-	if (detected_count == 1)
-		return detected [0];
+	walk = _best_match (geom, detected, detected_error, detected_count);
+	if (walk)
+		return walk;
 	return _probe_with_open (geom, detected_count, detected);
 }
 
@@ -154,31 +232,39 @@ ped_file_system_clobber (PedGeometry* geom)
 
 	PED_ASSERT (geom != NULL, return 0);
 
+	if (!ped_device_open (geom->dev))
+		goto error;
+
 	ped_exception_fetch_all ();
 	while ((fs_type = ped_file_system_type_get_next (fs_type))) {
-		int		probbed;
-		PED_ASSERT (fs_type->ops->clobber != NULL, return 0);
-		PED_ASSERT (fs_type->ops->probe != NULL, return 0);
+		PedGeometry*	probed;
 
-		probbed = fs_type->ops->probe (geom);
-		if (!probbed) {
+		PED_ASSERT (fs_type->ops->clobber != NULL, return 0);
+
+		probed = ped_file_system_probe_specific (fs_type, geom);
+		if (!probed) {
 			ped_exception_catch ();
 			continue;
 		}
+		ped_geometry_destroy (probed);
 
-		if (!fs_type->ops->clobber (geom))
-			goto error;
+		if (fs_type->ops->clobber && !fs_type->ops->clobber (geom)) {
+			ped_exception_leave_all ();
+			goto error_close_dev;
+		}
 	}
+	ped_device_close (geom->dev);
 	ped_exception_leave_all ();
 	return 1;
 
+error_close_dev:
+	ped_device_close (geom->dev);
 error:
-	ped_exception_leave_all ();
 	return 0;
 }
 
 static int
-ped_file_system_clobber_exclude (const PedGeometry* geom,
+ped_file_system_clobber_exclude (PedGeometry* geom,
 				 const PedGeometry* exclude)
 {
 	PedGeometry*    clobber_geom;
@@ -200,29 +286,62 @@ PedFileSystem*
 ped_file_system_open (PedGeometry* geom)
 {
 	PedFileSystemType*	type;
+	PedFileSystem*		fs;
+	PedGeometry*		probed_geom;
 
 	PED_ASSERT (geom != NULL, return NULL);
 
+	if (!ped_device_open (geom->dev))
+		goto error;
+
 	type = ped_file_system_probe (geom);
 	if (!type) {
 		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
 				     _("Could not detect file system."));
-		return NULL;
+		goto error_close_dev;
 	}
+
+	probed_geom = ped_file_system_probe_specific (type, geom);
+	if (!probed_geom)
+		goto error_close_dev;
+	if (!ped_geometry_test_inside (geom, probed_geom)) {
+		if (ped_exception_throw (
+			PED_EXCEPTION_ERROR,
+			PED_EXCEPTION_IGNORE_CANCEL,
+			_("The file system is bigger than it's volume!"))
+				!= PED_EXCEPTION_IGNORE)
+			goto error_destroy_probed_geom;
+	}
+
 	if (!type->ops->open) {
 		ped_exception_throw (PED_EXCEPTION_NO_FEATURE,
 				     PED_EXCEPTION_CANCEL,
 				     _("Support for opening %s file systems "
 				       "is not implemented yet."),
 				     type->name);
-		return NULL;
+		goto error_destroy_probed_geom;
 	}
-	return type->ops->open (geom);
+
+	fs = type->ops->open (probed_geom);
+	if (!fs)
+		goto error_destroy_probed_geom;
+	ped_geometry_destroy (probed_geom);
+	return fs;
+
+error_destroy_probed_geom:
+	ped_geometry_destroy (probed_geom);
+error_close_dev:
+	ped_device_close (geom->dev);
+error:
+	return 0;
 }
 
 PedFileSystem*
-ped_file_system_create (PedGeometry* geom, const PedFileSystemType* type)
+ped_file_system_create (PedGeometry* geom, const PedFileSystemType* type,
+			PedTimer* timer)
 {
+	PedFileSystem*	fs;
+
 	PED_ASSERT (geom != NULL, return NULL);
 	PED_ASSERT (type != NULL, return NULL);
 
@@ -232,24 +351,45 @@ ped_file_system_create (PedGeometry* geom, const PedFileSystemType* type)
 				     _("Support for creating %s file systems "
 				       "is not implemented yet."),
 				     type->name);
-		return NULL;
+		goto error;
 	}
 
-	if (!ped_file_system_clobber (geom))
-		return NULL;
+	if (!ped_device_open (geom->dev))
+		goto error;
 
-	return type->ops->create (geom);
+	if (!ped_file_system_clobber (geom))
+		goto error_close_dev;
+	fs = type->ops->create (geom, timer);
+	if (!fs)
+		goto error_close_dev;
+	return fs;
+
+error_close_dev:
+	ped_device_close (geom->dev);
+error:
+	return 0;
 }
 
 int
 ped_file_system_close (PedFileSystem* fs)
 {
-	PED_ASSERT (fs != NULL, return 0);
-	return fs->type->ops->close (fs);
+	PedDevice*	dev = fs->geom->dev;
+
+	PED_ASSERT (fs != NULL, goto error_close_dev);
+
+	if (!fs->type->ops->close (fs))
+		goto error_close_dev;
+	ped_device_close (dev);
+	return 1;
+
+error_close_dev:
+	ped_device_close (dev);
+error:
+	return 0;
 }
 
 int
-ped_file_system_check (PedFileSystem* fs)
+ped_file_system_check (PedFileSystem* fs, PedTimer* timer)
 {
 	PED_ASSERT (fs != NULL, return 0);
 
@@ -261,11 +401,11 @@ ped_file_system_check (PedFileSystem* fs)
 				     fs->type->name);
 		return 0;
 	}
-	return fs->type->ops->check (fs);
+	return fs->type->ops->check (fs, timer);
 }
 
 static int
-_raw_copy (const PedGeometry* src, PedGeometry* dest)
+_raw_copy (const PedGeometry* src, PedGeometry* dest, PedTimer* timer)
 {
 	char*		buf;
 	PedSector	pos;
@@ -278,73 +418,108 @@ _raw_copy (const PedGeometry* src, PedGeometry* dest)
 	if (!buf)
 		goto error;
 
+	if (!ped_device_open (src->dev))
+		goto error_free_buf;
+	if (!ped_device_open (dest->dev))
+		goto error_close_src;
+
 	for (pos = 0; pos + BUFFER_SIZE < src->length; pos += BUFFER_SIZE) {
+		ped_timer_update (timer, 1.0 * pos / src->length);
 		if (!ped_geometry_read (src, buf, pos, BUFFER_SIZE))
-			goto error_free_buf;
+			goto error_close_dest;
 		if (!ped_geometry_write (dest, buf, pos, BUFFER_SIZE))
-			goto error_free_buf;
+			goto error_close_dest;
 	}
-
 	if (pos < src->length) {
+		ped_timer_update (timer, 1.0 * pos / src->length);
 		if (!ped_geometry_read (src, buf, pos, src->length - pos))
-			goto error_free_buf;
+			goto error_close_dest;
 		if (!ped_geometry_write (dest, buf, pos, src->length - pos))
-			goto error_free_buf;
+			goto error_close_dest;
 	}
+	ped_timer_update (timer, 1.0);
 
+	ped_device_close (src->dev);
+	ped_device_close (dest->dev);
 	ped_free (buf);
 	return 1;
 
+error_close_dest:
+	ped_device_close (dest->dev);
+error_close_src:
+	ped_device_close (src->dev);
 error_free_buf:
 	ped_free (buf);
 error:
 	return 0;
 }
 
-static int
-_raw_copy_and_resize (const PedFileSystem* fs, PedGeometry* geom)
+static PedFileSystem*
+_raw_copy_and_resize (const PedFileSystem* fs, PedGeometry* geom,
+		      PedTimer* timer)
 {
-	PedFileSystem*		new_fs;
+	PedFileSystem*	new_fs;
+	PedTimer*	sub_timer = NULL;
 
-	if (!_raw_copy (fs->geom, geom))
+	ped_timer_reset (timer);
+	ped_timer_set_state_name (timer, _("raw block copying"));
+
+	sub_timer = ped_timer_new_nested (timer, 0.95);
+	if (!_raw_copy (fs->geom, geom, sub_timer))
 		goto error;
+	ped_timer_destroy_nested (sub_timer);
 
 	new_fs = ped_file_system_open (geom);
 	if (!new_fs)
 		goto error;
-	if (!ped_file_system_resize (new_fs, geom))
+
+	ped_timer_set_state_name (timer, _("growing file system"));
+
+	sub_timer = ped_timer_new_nested (timer, 0.05);
+	if (!ped_file_system_resize (new_fs, geom, sub_timer))
 		goto error_close_new_fs;
-	ped_file_system_close (new_fs);
-	return 1;
+	ped_timer_destroy_nested (sub_timer);
+	return new_fs;
 
 error_close_new_fs:
 	ped_file_system_close (new_fs);
 error:
-	return 0;
+	ped_timer_destroy_nested (sub_timer);
+	return NULL;
 }
 
-int
-ped_file_system_copy (const PedFileSystem* fs, PedGeometry* geom)
+PedFileSystem*
+ped_file_system_copy (PedFileSystem* fs, PedGeometry* geom, PedTimer* timer)
 {
+	PedFileSystem* new_fs;
+
 	PED_ASSERT (fs != NULL, return 0);
 	PED_ASSERT (geom != NULL, return 0);
 
+	if (!ped_device_open (geom->dev))
+		goto error;
+
 	if (ped_geometry_test_overlap (fs->geom, geom)) {
 		ped_exception_throw (
 			PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
 			_("Can't copy onto an overlapping partition."));
-		return 0;
+		goto error_close_dev;
+	}
+
+	if (!fs->checked) {
+		if (!ped_file_system_check (fs, timer))
+			goto error_close_dev;
 	}
 
 	if (!ped_file_system_clobber_exclude (geom, fs->geom))
-		return 0;
+		goto error_close_dev;
 
 	if (!fs->type->ops->copy) {
 		if (fs->type->ops->resize) {
 			if (fs->geom->length <= geom->length)
 				return _raw_copy_and_resize (
-						fs,
-					       	(PedGeometry*) geom);
+						fs, (PedGeometry*) geom,
+						timer);
 				
 			ped_exception_throw (
 				PED_EXCEPTION_NO_FEATURE,
@@ -358,7 +533,7 @@ ped_file_system_copy (const PedFileSystem* fs, PedGeometry* geom)
 				  "you are trying to copy, or copy to a bigger "
 				  "partition."),
 				fs->type->name);
-			return 0;
+			goto error_close_dev;
 		} else {
 			ped_exception_throw (
 				PED_EXCEPTION_NO_FEATURE,
@@ -366,14 +541,22 @@ ped_file_system_copy (const PedFileSystem* fs, PedGeometry* geom)
 				_("Support for copying %s file systems is not "
 				  "implemented yet."),
 				fs->type->name);
-			return 0;
+			goto error_close_dev;
 		}
 	}
-	return fs->type->ops->copy (fs, geom);
+	new_fs = fs->type->ops->copy (fs, geom, timer);
+	if (!new_fs)
+		goto error_close_dev;
+	return new_fs;
+
+error_close_dev:
+	ped_device_close (geom->dev);
+error:
+	return NULL;;
 }
 
 int
-ped_file_system_resize (PedFileSystem* fs, const PedGeometry* geom)
+ped_file_system_resize (PedFileSystem* fs, PedGeometry* geom, PedTimer* timer)
 {
 	PED_ASSERT (fs != NULL, return 0);
 	PED_ASSERT (geom != NULL, return 0);
@@ -386,9 +569,26 @@ ped_file_system_resize (PedFileSystem* fs, const PedGeometry* geom)
 				     fs->type->name);
 		return 0;
 	}
+	if (!fs->checked) {
+		if (!ped_file_system_check (fs, timer))
+			return 0;
+	}
 	if (!ped_file_system_clobber_exclude (geom, fs->geom))
 		return 0;
-	return fs->type->ops->resize (fs, geom);
+
+	return fs->type->ops->resize (fs, geom, timer);
+}
+
+PedConstraint*
+ped_file_system_get_create_constraint (const PedFileSystemType* fs_type,
+				       const PedDevice* dev)
+{
+	PED_ASSERT (fs_type != NULL, return NULL);
+	PED_ASSERT (dev != NULL, return NULL);
+
+	if (!fs_type->ops->get_create_constraint)
+		return NULL;
+	return fs_type->ops->get_create_constraint (dev);
 }
 
 PedConstraint*
@@ -396,35 +596,31 @@ ped_file_system_get_resize_constraint (const PedFileSystem* fs)
 {
 	PED_ASSERT (fs != NULL, return 0);
 
-	if (!fs->type->ops->get_resize_constraint) {
-		ped_exception_throw (PED_EXCEPTION_NO_FEATURE,
-				     PED_EXCEPTION_CANCEL,
-				     _("No get_resize_constraint for %s!"),
-				     fs->type->name);
-		return 0;
-	}
+	if (!fs->type->ops->get_resize_constraint)
+		return NULL;
 	return fs->type->ops->get_resize_constraint (fs);
 }
 
-int
-ped_file_system_set_system (const PedFileSystem* fs, PedPartition* part,
-			    const PedDiskType* disk_type)
+PedConstraint*
+ped_file_system_get_copy_constraint (const PedFileSystem* fs,
+				     const PedDevice* dev)
 {
-	PED_ASSERT (fs != NULL, return 0);
-	PED_ASSERT (part != NULL, return 0);
-	PED_ASSERT (disk_type != NULL, return 0);
-	PED_ASSERT (fs->type->ops->set_system != NULL, return 0);
+	PedGeometry	full_dev;
 
-	if (!fs->type->ops->set_system (fs, part, disk_type)) {
-		return ped_exception_throw (
-			PED_EXCEPTION_NO_FEATURE,
-			PED_EXCEPTION_IGNORE_CANCEL,
-			_("The %s file system code doesn't support %s disk "
-			  "labels."),
-			fs->type->name, disk_type->name)
-				== PED_EXCEPTION_IGNORE;
+	PED_ASSERT (fs != NULL, return NULL);
+	PED_ASSERT (dev != NULL, return NULL);
+
+	if (fs->type->ops->get_copy_constraint)
+		return fs->type->ops->get_copy_constraint (fs, dev);
+
+	if (fs->type->ops->resize) {
+		if (!ped_geometry_init (&full_dev, dev, 0, dev->length - 1))
+			return NULL;
+		return ped_constraint_new (
+				ped_alignment_any, ped_alignment_any,
+				&full_dev, &full_dev,
+				fs->geom->length, dev->length);
 	}
 
-	return 1;
+	return NULL;
 }
-
