@@ -17,10 +17,10 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-static const char _ext2_c[] = "$Id: ext2.c,v 1.54 1999/10/08 06:20:16 buytenh Exp $";
-
 #include "config.h"
 
+#ifndef DISCOVER_ONLY
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -257,26 +257,6 @@ error_free_buf:
 error:
 	return 0;
 }
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
 
 off_t ext2_get_inode_offset(struct ext2_fs *fs, ino_t inode, blk_t *block)
 {
@@ -369,7 +349,7 @@ static void
 _inode_update_size(struct ext2_fs *fs, struct ext2_inode *inode, int delta)
 {
 	int		i512perblock = 1 << (fs->logsize - 9);
-	__u64		size;
+	uint64_t	size;
 
 	/* i_blocks is in 512 byte blocks */
 	inode->i_blocks = PED_CPU_TO_LE32(EXT2_INODE_BLOCKS(*inode)
@@ -384,7 +364,7 @@ int ext2_do_inode(struct ext2_fs *fs, struct ext2_inode *inode, blk_t block,
 		   int action)
 {
 	struct ext2_buffer_head *bh;
-	__u32			*udata;
+	uint32_t		*udata;
 	blk_t			 count = 0;
 	int			 i;
 	int			 u32perblock = fs->blocksize >> 2;
@@ -443,7 +423,7 @@ int ext2_do_inode(struct ext2_fs *fs, struct ext2_inode *inode, blk_t block,
 		return -1;
 
 	bh = ext2_bread(fs, EXT2_INODE_BLOCK(*inode, EXT2_IND_BLOCK));
-	udata = (__u32 *)bh->data;
+	udata = (uint32_t *)bh->data;
 
 	/* Indirect blocks for next 256/512/1024 blocks (for 1k/2k/4k blocks) */
 	for (i = 0; i < u32perblock; i++) {
@@ -480,12 +460,12 @@ int ext2_do_inode(struct ext2_fs *fs, struct ext2_inode *inode, blk_t block,
 	    (count >= EXT2_INODE_BLOCKS(*inode) && action != EXT2_ACTION_ADD))
 		return -1;
 	bh = ext2_bread(fs, EXT2_INODE_BLOCK(*inode, EXT2_DIND_BLOCK));
-	udata = (__u32 *)bh->data;
+	udata = (uint32_t *)bh->data;
 
 	/* Double indirect blocks for next 2^16/2^18/2^20 1k/2k/4k blocks */
 	for (i = 0; i < u32perblock; i++) {
 		struct ext2_buffer_head	*bh2;
-		__u32			*udata2;
+		uint32_t			*udata2;
 		int			 j;
 
 		if (!udata[i]) {
@@ -493,7 +473,7 @@ int ext2_do_inode(struct ext2_fs *fs, struct ext2_inode *inode, blk_t block,
 			return -1;
 		}
 		bh2 = ext2_bread(fs, PED_LE32_TO_CPU(udata[i]));
-		udata2 = (__u32 *)bh2->data;
+		udata2 = (uint32_t *)bh2->data;
 		count += i512perblock;
 
 		for (j = 0; j < u32perblock; j++) {
@@ -596,15 +576,7 @@ int ext2_is_group_sparse(struct ext2_fs *fs, int group)
 
 	return 0;
 }
-
-
-
-
-
-
-
-
-
+
 void ext2_close(struct ext2_fs *fs)
 {
 	ext2_commit_metadata(fs, EXT2_META_PRIMARY | EXT2_META_BACKUP);
@@ -624,6 +596,7 @@ int ext2_commit_metadata(struct ext2_fs *fs, int copies)
 	int		num;
 	int		wmeta = fs->metadirty & copies;
 	unsigned char	sb[fs->blocksize];
+	struct ext2_super_block *sb_for_io;
 	int		sb_block;
 
 	/* See if there is even anything to write... */
@@ -637,10 +610,13 @@ int ext2_commit_metadata(struct ext2_fs *fs, int copies)
 	if (!ext2_read_blocks (fs, sb, 0, 1))
 		return 0;
 
-	if (EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb))
+	if (EXT2_SUPER_FIRST_DATA_BLOCK(fs->sb)) {
 		memcpy(sb, &fs->sb, 1024);
-	else
+		sb_for_io = (struct ext2_super_block *) sb;
+	} else {
 		memcpy(sb+1024, &fs->sb, 1024);
+		sb_for_io = (struct ext2_super_block *) (sb + 1024);
+	}
 
 	num = copies & EXT2_META_BACKUP ? fs->numgroups : 1;
 
@@ -651,6 +627,9 @@ int ext2_commit_metadata(struct ext2_fs *fs, int copies)
 		if (!ext2_is_group_sparse(fs, i))
 			continue;
 
+		if (fs->dynamic_version)
+			sb_for_io->s_block_group_nr = PED_CPU_TO_LE16 (i);
+
 		if ((i == 0 && wmeta & EXT2_META_PRIMARY_SB) ||
 		    (i != 0 && wmeta & EXT2_META_SB))
 		{
@@ -671,6 +650,8 @@ int ext2_commit_metadata(struct ext2_fs *fs, int copies)
 		}
 	}
 
+	sb_for_io->s_block_group_nr = 0;
+
 	/* Clear the flags of the components we just finished writing. */
 	fs->metadirty &= ~copies;
 
@@ -713,8 +694,7 @@ static int ext2_determine_itoffset(struct ext2_fs *fs)
 			ped_exception_throw (PED_EXCEPTION_NO_FEATURE,
 				PED_EXCEPTION_CANCEL,
 			_("This ext2 filesystem has a rather strange layout!  "
-			  "Please use dumpe2fs on this filesystem and send it "
-			  "to <buytenh@gnu.org>. I won't resize it, sorry."));
+			  "Parted can't resize this (yet)."));
 			return 0;
 		}
 	}
@@ -749,7 +729,7 @@ struct ext2_fs *ext2_open(struct ext2_dev_handle *handle, int state)
 		goto error_free_fs;
 	}
 
-	
+
 	fs->opt_debug = 1;
 	fs->opt_safe = 1;
 	fs->opt_verbose = 0;
@@ -765,7 +745,7 @@ struct ext2_fs *ext2_open(struct ext2_dev_handle *handle, int state)
 
 	if (!((EXT2_SUPER_STATE(fs->sb) | state) & EXT2_VALID_FS)
 	    || (EXT2_SUPER_FEATURE_INCOMPAT(fs->sb)
-	    	& EXT3_FEATURE_INCOMPAT_RECOVER))
+		& EXT3_FEATURE_INCOMPAT_RECOVER))
 	{
 		if (ped_exception_throw (
 			PED_EXCEPTION_WARNING, PED_EXCEPTION_IGNORE_CANCEL,
@@ -775,14 +755,16 @@ struct ext2_fs *ext2_open(struct ext2_dev_handle *handle, int state)
 			goto error_free_fs;
 	}
 
+	fs->dynamic_version = EXT2_SUPER_REV_LEVEL (fs->sb) > 0;
+
 	if ((EXT2_SUPER_FEATURE_COMPAT(fs->sb)
 	                & ~EXT3_FEATURE_COMPAT_HAS_JOURNAL) ||
 	    (EXT2_SUPER_FEATURE_INCOMPAT(fs->sb)
 	    		& ~(EXT2_FEATURE_INCOMPAT_FILETYPE |
 			    EXT3_FEATURE_INCOMPAT_RECOVER)) ||
 	    (EXT2_SUPER_FEATURE_RO_COMPAT(fs->sb)
-	                & ~(EXT2_FEATURE_RO_COMPAT_SPARSE_SUPER |
-	    		    EXT2_FEATURE_RO_COMPAT_LARGE_FILE)))
+			& ~(EXT2_FEATURE_RO_COMPAT_SPARSE_SUPER |
+			    EXT2_FEATURE_RO_COMPAT_LARGE_FILE)))
 	{
 		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
 		     _("Filesystem has incompatible feature enabled"));
@@ -820,7 +802,7 @@ struct ext2_fs *ext2_open(struct ext2_dev_handle *handle, int state)
 		fs->sparse = 1;
 
 	fs->has_journal = 0 < (EXT2_SUPER_FEATURE_COMPAT(fs->sb)
-	    			& EXT3_FEATURE_COMPAT_HAS_JOURNAL);
+			       & EXT3_FEATURE_COMPAT_HAS_JOURNAL);
 	fs->has_internal_journal
 		= fs->has_journal
 			&& uuid_is_null(EXT2_SUPER_JOURNAL_UUID(fs->sb))
@@ -838,7 +820,6 @@ struct ext2_fs *ext2_open(struct ext2_dev_handle *handle, int state)
 		goto error_free_gd;
 
 	fs->metadirty = 0;
-
 	return fs;
 
 error_free_gd:
@@ -850,3 +831,5 @@ error_free_fs:
 error:
 	return NULL;
 }
+
+#endif /* !DISCOVER_ONLY */
