@@ -21,9 +21,8 @@
 #ifndef _EXT2_FS_H
 #define _EXT2_FS_H
 
-static const char _ext2_fs_h[] = "$Id: ext2_fs.h,v 1.3 1999/12/31 09:36:41 buytenh Exp $";
-
 #include <parted/endian.h>
+#include <stdint.h>
 
 /*
  * The second extended filesystem constants/structures
@@ -63,7 +62,7 @@ static const char _ext2_fs_h[] = "$Id: ext2_fs.h,v 1.3 1999/12/31 09:36:41 buyte
 /*
  * Special inodes numbers
  */
-#define	EXT2_BAD_INO		 1	/* Bad blocks inode */
+#define EXT2_BAD_INO		 1	/* Bad blocks inode */
 #define EXT2_ROOT_INO		 2	/* Root inode */
 #define EXT2_ACL_IDX_INO	 3	/* ACL inode */
 #define EXT2_ACL_DATA_INO	 4	/* ACL inode */
@@ -93,74 +92,74 @@ static const char _ext2_fs_h[] = "$Id: ext2_fs.h,v 1.3 1999/12/31 09:36:41 buyte
 
 struct ext2_dir_entry_2
 {
-	__u32	inode;
-	__u16	rec_len;
-	__u8	name_len;
-	__u8	file_type;
-	char	name[255];
+	uint32_t	inode;
+	uint16_t	rec_len;
+	uint8_t		name_len;
+	uint8_t		file_type;
+	char		name[255];
 };
 
 struct ext2_group_desc
 {
-	__u32	bg_block_bitmap;
-	__u32	bg_inode_bitmap;
-	__u32	bg_inode_table;
-	__u16	bg_free_blocks_count;
-	__u16	bg_free_inodes_count;
-	__u16	bg_used_dirs_count;
-	__u16	bg_pad;
-	__u32	bg_reserved[3];
+	uint32_t	bg_block_bitmap;
+	uint32_t	bg_inode_bitmap;
+	uint32_t	bg_inode_table;
+	uint16_t	bg_free_blocks_count;
+	uint16_t	bg_free_inodes_count;
+	uint16_t	bg_used_dirs_count;
+	uint16_t	bg_pad;
+	uint32_t	bg_reserved[3];
 };
 
 struct ext2_inode
 {
-	__u16	i_mode;		/* File mode */
-	__u16	i_uid;		/* Owner Uid */
-	__u32	i_size;		/* Size in bytes */
-	__u32	i_atime;	/* Access time */
-	__u32	i_ctime;	/* Creation time */
-	__u32	i_mtime;	/* Modification time */
-	__u32	i_dtime;	/* Deletion Time */
-	__u16	i_gid;		/* Group Id */
-	__u16	i_links_count;	/* Links count */
-	__u32	i_blocks;	/* Blocks count */
-	__u32	i_flags;	/* File flags */
+	uint16_t	i_mode;		/* File mode */
+	uint16_t	i_uid;		/* Owner Uid */
+	uint32_t	i_size;		/* Size in bytes */
+	uint32_t	i_atime;	/* Access time */
+	uint32_t	i_ctime;	/* Creation time */
+	uint32_t	i_mtime;	/* Modification time */
+	uint32_t	i_dtime;	/* Deletion Time */
+	uint16_t	i_gid;		/* Group Id */
+	uint16_t	i_links_count;	/* Links count */
+	uint32_t	i_blocks;	/* Blocks count */
+	uint32_t	i_flags;	/* File flags */
 	union {
 		struct {
-			__u32	l_i_reserved1;
+			uint32_t	l_i_reserved1;
 		} linux1;
 		struct {
-			__u32	h_i_translator;
+			uint32_t	h_i_translator;
 		} hurd1;
 		struct {
-			__u32	m_i_reserved1;
+			uint32_t	m_i_reserved1;
 		} masix1;
 	} osd1;			/* OS dependent 1 */
-	__u32	i_block[EXT2_N_BLOCKS];/* Pointers to blocks */
-	__u32	i_generation;	/* File version (for NFS) */
-	__u32	i_file_acl;	/* File ACL */
-	__u32	i_dir_acl;	/* Directory ACL */
-	__u32	i_faddr;	/* Fragment address */
+	uint32_t	i_block[EXT2_N_BLOCKS];/* Pointers to blocks */
+	uint32_t	i_generation;	/* File version (for NFS) */
+	uint32_t	i_file_acl;	/* File ACL */
+	uint32_t	i_dir_acl;	/* Directory ACL */
+	uint32_t	i_faddr;	/* Fragment address */
 	union {
 		struct {
-			__u8	l_i_frag;	/* Fragment number */
-			__u8	l_i_fsize;	/* Fragment size */
-			__u16	i_pad1;
-			__u32	l_i_reserved2[2];
+			uint8_t		l_i_frag;	/* Fragment number */
+			uint8_t		l_i_fsize;	/* Fragment size */
+			uint16_t	i_pad1;
+			uint32_t	l_i_reserved2[2];
 		} linux2;
 		struct {
-			__u8	h_i_frag;	/* Fragment number */
-			__u8	h_i_fsize;	/* Fragment size */
-			__u16	h_i_mode_high;
-			__u16	h_i_uid_high;
-			__u16	h_i_gid_high;
-			__u32	h_i_author;
+			uint8_t		h_i_frag;	/* Fragment number */
+			uint8_t		h_i_fsize;	/* Fragment size */
+			uint16_t	h_i_mode_high;
+			uint16_t	h_i_uid_high;
+			uint16_t	h_i_gid_high;
+			uint32_t	h_i_author;
 		} hurd2;
 		struct {
-			__u8	m_i_frag;	/* Fragment number */
-			__u8	m_i_fsize;	/* Fragment size */
-			__u16	m_pad1;
-			__u32	m_i_reserved2[2];
+			uint8_t		m_i_frag;	/* Fragment number */
+			uint8_t		m_i_fsize;	/* Fragment size */
+			uint16_t	m_pad1;
+			uint32_t	m_i_reserved2[2];
 		} masix2;
 	} osd2;					/* OS dependent 2 */
 };
@@ -169,31 +168,31 @@ struct ext2_inode
 
 struct ext2_super_block
 {
-	__u32	s_inodes_count;		/* Inodes count */
-	__u32	s_blocks_count;		/* Blocks count */
-	__u32	s_r_blocks_count;	/* Reserved blocks count */
-	__u32	s_free_blocks_count;	/* Free blocks count */
-	__u32	s_free_inodes_count;	/* Free inodes count */
-	__u32	s_first_data_block;	/* First Data Block */
-	__u32	s_log_block_size;	/* Block size */
-	__s32	s_log_frag_size;	/* Fragment size */
-	__u32	s_blocks_per_group;	/* # Blocks per group */
-	__u32	s_frags_per_group;	/* # Fragments per group */
-	__u32	s_inodes_per_group;	/* # Inodes per group */
-	__u32	s_mtime;		/* Mount time */
-	__u32	s_wtime;		/* Write time */
-	__u16	s_mnt_count;		/* Mount count */
-	__s16	s_max_mnt_count;	/* Maximal mount count */
-	__u16	s_magic;		/* Magic signature */
-	__u16	s_state;		/* File system state */
-	__u16	s_errors;		/* Behaviour when detecting errors */
-	__u16	s_minor_rev_level;	/* minor revision level */
-	__u32	s_lastcheck;		/* time of last check */
-	__u32	s_checkinterval;	/* max. time between checks */
-	__u32	s_creator_os;		/* OS */
-	__u32	s_rev_level;		/* Revision level */
-	__u16	s_def_resuid;		/* Default uid for reserved blocks */
-	__u16	s_def_resgid;		/* Default gid for reserved blocks */
+	uint32_t	s_inodes_count;		/* Inodes count */
+	uint32_t	s_blocks_count;		/* Blocks count */
+	uint32_t	s_r_blocks_count;	/* Reserved blocks count */
+	uint32_t	s_free_blocks_count;	/* Free blocks count */
+	uint32_t	s_free_inodes_count;	/* Free inodes count */
+	uint32_t	s_first_data_block;	/* First Data Block */
+	uint32_t	s_log_block_size;	/* Block size */
+	int32_t		s_log_frag_size;	/* Fragment size */
+	uint32_t	s_blocks_per_group;	/* # Blocks per group */
+	uint32_t	s_frags_per_group;	/* # Fragments per group */
+	uint32_t	s_inodes_per_group;	/* # Inodes per group */
+	uint32_t	s_mtime;		/* Mount time */
+	uint32_t	s_wtime;		/* Write time */
+	uint16_t	s_mnt_count;		/* Mount count */
+	int16_t		s_max_mnt_count;	/* Maximal mount count */
+	uint16_t	s_magic;		/* Magic signature */
+	uint16_t	s_state;		/* File system state */
+	uint16_t	s_errors;		/* Behaviour when detecting errors */
+	uint16_t	s_minor_rev_level;	/* minor revision level */
+	uint32_t	s_lastcheck;		/* time of last check */
+	uint32_t	s_checkinterval;	/* max. time between checks */
+	uint32_t	s_creator_os;		/* OS */
+	uint32_t	s_rev_level;		/* Revision level */
+	uint16_t	s_def_resuid;		/* Default uid for reserved blocks */
+	uint16_t	s_def_resgid;		/* Default gid for reserved blocks */
 	/*
 	 * These fields are for EXT2_DYNAMIC_REV superblocks only.
 	 *
@@ -207,32 +206,32 @@ struct ext2_super_block
 	 * feature set, it must abort and not try to meddle with
 	 * things it doesn't understand...
 	 */
-	__u32	s_first_ino;		/* First non-reserved inode */
-	__u16	s_inode_size;		/* size of inode structure */
-	__u16	s_block_group_nr;	/* block group # of this superblock */
-	__u32	s_feature_compat;	/* compatible feature set */
-	__u32	s_feature_incompat;	/* incompatible feature set */
-	__u32	s_feature_ro_compat;	/* readonly-compatible feature set */
-	__u8	s_uuid[16];		/* 128-bit uuid for volume */
-	char	s_volume_name[16];	/* volume name */
-	char	s_last_mounted[64];	/* directory where last mounted */
-	__u32	s_algorithm_usage_bitmap;  /* For compression */
+	uint32_t	s_first_ino;		/* First non-reserved inode */
+	uint16_t	s_inode_size;		/* size of inode structure */
+	uint16_t	s_block_group_nr;	/* block group # of this superblock */
+	uint32_t	s_feature_compat;	/* compatible feature set */
+	uint32_t	s_feature_incompat;	/* incompatible feature set */
+	uint32_t	s_feature_ro_compat;	/* readonly-compatible feature set */
+	uint8_t		s_uuid[16];		/* 128-bit uuid for volume */
+	char		s_volume_name[16];	/* volume name */
+	char		s_last_mounted[64];	/* directory where last mounted */
+	uint32_t	s_algorithm_usage_bitmap;  /* For compression */
 	/*
 	 * Performance hints.  Directory preallocation should only
 	 * happen if the EXT2_COMPAT_PREALLOC flag is on.
 	 */
-	__u8	s_prealloc_blocks;	/* Nr of blocks to try to preallocate*/
-	__u8	s_prealloc_dir_blocks;	/* Nr to preallocate for dirs */
-	__u16	s_padding1;
+	uint8_t		s_prealloc_blocks;	/* Nr of blocks to try to preallocate*/
+	uint8_t		s_prealloc_dir_blocks;	/* Nr to preallocate for dirs */
+	uint16_t	s_padding1;
 	/*
 	 * Journaling support valid if EXT2_FEATURE_COMPAT_HAS_JOURNAL set.
 	 */
-	__u8	s_journal_uuid[16];	/* uuid of journal superblock */
-	__u32	s_journal_inum;		/* inode number of journal file */
-	__u32	s_journal_dev;		/* device number of journal file */
-	__u32	s_last_orphan;		/* start of list of inodes to delete */
+	uint8_t		s_journal_uuid[16];	/* uuid of journal superblock */
+	uint32_t	s_journal_inum;		/* inode number of journal file */
+	uint32_t	s_journal_dev;		/* device number of journal file */
+	uint32_t	s_last_orphan;		/* start of list of inodes to delete */
 
-	__u32	s_reserved[197];	/* Padding to the end of the block */
+	uint32_t	s_reserved[197];	/* Padding to the end of the block */
 };
 
 #define EXT2_DIRENT_INODE(dir_ent)	(PED_LE32_TO_CPU((dir_ent).inode))
@@ -253,8 +252,8 @@ struct ext2_super_block
 #define EXT2_INODE_MODE(inode)		(PED_LE16_TO_CPU((inode).i_mode))
 #define EXT2_INODE_UID(inode)		(PED_LE16_TO_CPU((inode).i_uid))
 #define EXT2_INODE_SIZE(inode) \
-	((__u64) PED_LE32_TO_CPU((inode).i_size) \
-	 + ((__u64) PED_LE32_TO_CPU((inode).i_size_high) << 32))
+	((uint64_t) PED_LE32_TO_CPU((inode).i_size) \
+	+ ((uint64_t) PED_LE32_TO_CPU((inode).i_size_high) << 32))
 #define EXT2_INODE_ATIME(inode)		(PED_LE32_TO_CPU((inode).i_atime))
 #define EXT2_INODE_CTIME(inode)		(PED_LE32_TO_CPU((inode).i_ctime))
 #define EXT2_INODE_MTIME(inode)		(PED_LE32_TO_CPU((inode).i_mtime))
@@ -277,7 +276,7 @@ struct ext2_super_block
 		(PED_LE32_TO_CPU((sb).s_first_data_block))
 #define EXT2_SUPER_LOG_BLOCK_SIZE(sb)	(PED_LE32_TO_CPU((sb).s_log_block_size))
 #define EXT2_SUPER_LOG_FRAG_SIZE(sb) \
-		((__s32) PED_LE32_TO_CPU((sb).s_log_frag_size))
+		((int32_t) PED_LE32_TO_CPU((sb).s_log_frag_size))
 #define EXT2_SUPER_BLOCKS_PER_GROUP(sb)	\
 		(PED_LE32_TO_CPU((sb).s_blocks_per_group))
 #define EXT2_SUPER_FRAGS_PER_GROUP(sb) \
@@ -288,7 +287,7 @@ struct ext2_super_block
 #define EXT2_SUPER_WTIME(sb)		(PED_LE32_TO_CPU((sb).s_wtime))
 #define EXT2_SUPER_MNT_COUNT(sb)	(PED_LE16_TO_CPU((sb).s_mnt_count))
 #define EXT2_SUPER_MAX_MNT_COUNT(sb) \
-		((__s16) PED_LE16_TO_CPU((sb).s_max_mnt_count))
+		((int16_t) PED_LE16_TO_CPU((sb).s_max_mnt_count))
 #define EXT2_SUPER_MAGIC(sb)		(PED_LE16_TO_CPU((sb).s_magic))
 #define EXT2_SUPER_STATE(sb)		(PED_LE16_TO_CPU((sb).s_state))
 #define EXT2_SUPER_ERRORS(sb)		(PED_LE16_TO_CPU((sb).s_errors))
