@@ -21,142 +21,75 @@
  * merged 1.1.11 changes (by Andrew)
  */
 
-static const char _interface_c[] = "$Id: interface.c,v 1.6 1999/10/02 20:56:47 aclausen Exp $";
-
 #include "config.h"
 
-#include <parted/parted.h>
-#include <parted/disk_dos.h>
-#include <parted/disk_loop.h>
-#include <parted/disk_pc98.h>
-#include <parted/disk_mac.h>
-#include <parted/disk_bsd.h>
-#include <parted/disk_gpt.h>
-#include <parted/disk_sun.h>
-
+#include <stdlib.h>
 #include <string.h>
-
+#include <parted/parted.h>
 #include "ext2.h"
 #include "parted_io.h"
 
+static PedFileSystemType _ext2_type;
+static PedFileSystemType _ext3_type;
+
 struct ext2_dev_handle* ext2_make_dev_handle_from_parted_geometry(PedGeometry* geom);
 
-
-
-static int
-_ext2_probe (const PedGeometry* geom);
-
-static int
-_ext3_probe (const PedGeometry* geom);
-
-static int
-_ext2_clobber (PedGeometry* geom);
-
-static PedFileSystem*
-_ext2_open (const PedGeometry *geom);
-
-static PedFileSystem*
-_ext2_create (PedGeometry *geom);
-
-static int
-_ext2_close (PedFileSystem *fs);
-
-static int
-_ext2_check (PedFileSystem *fs);
-
-static int
-_ext2_resize (PedFileSystem* fs, const PedGeometry* geom);
-
-static PedConstraint*
-_ext2_get_resize_constraint (const PedFileSystem* fs);
-
-static int
-_ext2_set_system (const PedFileSystem* fs, PedPartition* part,
-		  const PedDiskType* disk_type);
-
-
-
-
-static PedFileSystemOps _ext2_ops = {
-	probe:		_ext2_probe,
-	clobber:	_ext2_clobber,
-	open:		_ext2_open,
-	create:         _ext2_create,
-	close:		_ext2_close,
-	check:          _ext2_check,
-	resize:		_ext2_resize,
-	copy:           NULL,
-	get_resize_constraint:	_ext2_get_resize_constraint,
-	set_system:	_ext2_set_system
-};
-
-static PedFileSystemType _ext2_type = {
-       next:		 NULL,
-       ops:		 &_ext2_ops,
-       name:		 "ext2"
-};
-
-
-
-static PedFileSystemOps _ext3_ops = {
-	probe:		_ext3_probe,
-	clobber:	_ext2_clobber,
-	open:		_ext2_open,
-	create:         NULL,			/* FIXME */
-	close:		_ext2_close,
-	check:          _ext2_check,
-	resize:		_ext2_resize,
-	copy:           NULL,
-	get_resize_constraint:	_ext2_get_resize_constraint,
-	set_system:	_ext2_set_system
-};
-
-static PedFileSystemType _ext3_type = {
-       next:		 NULL,
-       ops:		 &_ext3_ops,
-       name:		 "ext3"
-};
-
-void ped_file_system_ext2_init ()
-{
-	ped_file_system_type_register (&_ext3_type);
-	ped_file_system_type_register (&_ext2_type);
-}
-
-void ped_file_system_ext2_done ()
-{
-	ped_file_system_type_unregister (&_ext3_type);
-	ped_file_system_type_unregister (&_ext2_type);
-}
-
-static int
-_ext2_probe (const PedGeometry* geom)
+static PedGeometry*
+_ext2_generic_probe (PedGeometry* geom, int expect_ext3)
 {
 	struct ext2_super_block sb;
 
-	if (!ped_geometry_read(geom, &sb, 2, 2)
-	     || EXT2_SUPER_MAGIC(sb) != EXT2_SUPER_MAGIC_CONST
-	     || EXT2_SUPER_FEATURE_COMPAT(sb) & EXT3_FEATURE_COMPAT_HAS_JOURNAL)
-		return 0;
+	if (!ped_geometry_read(geom, &sb, 2, 2))
+		return NULL;
 
-	return 1;
+	if (EXT2_SUPER_MAGIC(sb) == EXT2_SUPER_MAGIC_CONST) {
+		PedSector block_size = 1 << (EXT2_SUPER_LOG_BLOCK_SIZE(sb) + 1);
+		PedSector block_count = EXT2_SUPER_BLOCKS_COUNT(sb);
+		PedSector group_blocks = EXT2_SUPER_BLOCKS_PER_GROUP(sb);
+		PedSector group_nr = EXT2_SUPER_BLOCK_GROUP_NR(sb);
+		PedSector first_data_block = EXT2_SUPER_FIRST_DATA_BLOCK(sb);
+		int version = EXT2_SUPER_REV_LEVEL(sb);
+		int is_ext3 = (EXT2_SUPER_FEATURE_COMPAT(sb) 
+				& EXT3_FEATURE_COMPAT_HAS_JOURNAL) != 0;
+
+		if (expect_ext3 != is_ext3)
+			return NULL;
+
+		if (version > 0 && group_nr > 0) {
+			PedSector group_size = group_blocks * block_size;
+			PedSector start;
+			PedGeometry probe_geom;
+
+			start = geom->start
+					- group_blocks * group_nr
+					- first_data_block;
+
+			if (start < 0)
+				return NULL;
+			ped_geometry_init (&probe_geom, geom->dev,
+					   start, block_count * block_size);
+			return _ext2_generic_probe (&probe_geom, expect_ext3);
+		} else {
+			return ped_geometry_new (geom->dev, geom->start,
+						 block_count * block_size);
+		}
+	}
+	return NULL;
 }
 
-
-static int
-_ext3_probe (const PedGeometry* geom)
+static PedGeometry*
+_ext2_probe (PedGeometry* geom)
 {
-	struct ext2_super_block sb;
-
-	if (!ped_geometry_read(geom, &sb, 2, 2)
-	    || EXT2_SUPER_MAGIC(sb) != EXT2_SUPER_MAGIC_CONST)
-		return 0;
-	if (EXT2_SUPER_FEATURE_COMPAT(sb) & EXT3_FEATURE_COMPAT_HAS_JOURNAL)
-	        return 1;
-
-	return 0;
+	return _ext2_generic_probe (geom, 0);
 }
 
+static PedGeometry*
+_ext3_probe (PedGeometry* geom)
+{
+	return _ext2_generic_probe (geom, 1);
+}
+
+#ifndef DISCOVER_ONLY
 static int
 _ext2_clobber (PedGeometry* geom)
 {
@@ -172,7 +105,7 @@ _ext2_clobber (PedGeometry* geom)
 }
 
 static PedFileSystem*
-_ext2_open (const PedGeometry *geom)
+_ext2_open (PedGeometry* geom)
 {
 	PedFileSystem*		fs;
 	struct ext2_fs*		fs_info;
@@ -183,6 +116,7 @@ _ext2_open (const PedGeometry *geom)
 
 	fs->type = &_ext2_type;
 	fs->geom = ped_geometry_duplicate (geom);
+	fs->checked = 1;
 
 	handle = ext2_make_dev_handle_from_parted_geometry(fs->geom);
 	if (!handle) goto error_free_fs;
@@ -204,7 +138,7 @@ error:
 }
 
 static PedFileSystem*
-_ext2_create (PedGeometry *geom)
+_ext2_create (PedGeometry* geom, PedTimer* timer)
 {
 	PedFileSystem*		fs;
 	struct ext2_fs*		fs_info;
@@ -219,7 +153,7 @@ _ext2_create (PedGeometry *geom)
 	handle = ext2_make_dev_handle_from_parted_geometry(fs->geom);
 	if (!handle) goto error_free_fs;
 
-	fs_info = ext2_mkfs (handle, 0, 0, 0, 0, -1, -1);
+	fs_info = ext2_mkfs (handle, 0, 0, 0, 0, -1, -1, timer);
 	if (!fs_info) goto error_free_handle;
 
 	fs->type_specific = (void*) fs_info;
@@ -249,7 +183,7 @@ _ext2_close (PedFileSystem *fs)
 }
 
 static int
-_ext2_check (PedFileSystem *fs)
+_ext2_check (PedFileSystem *fs, PedTimer* timer)
 {
 	ped_exception_throw (PED_EXCEPTION_INFORMATION, PED_EXCEPTION_OK,
 		_("The ext2 filesystem passed a basic check.  For a more "
@@ -258,16 +192,12 @@ _ext2_check (PedFileSystem *fs)
 }
 
 static int
-_ext2_resize (PedFileSystem* fs, const PedGeometry* geom)
+_ext2_resize (PedFileSystem* fs, PedGeometry* geom, PedTimer* timer)
 {
 	struct ext2_fs* f;
 	PedSector	old_length = fs->geom->length;
 
-	if (fs->geom->disk->dev != geom->disk->dev)
-	{
-		fprintf(stderr, "huh? ungrokkable!\n");
-		return 0;
-	}
+	PED_ASSERT (fs->geom->dev == geom->dev, return 0);
 
 	if (fs->geom->start != geom->start)
 	{
@@ -277,14 +207,14 @@ _ext2_resize (PedFileSystem* fs, const PedGeometry* geom)
 		return 0;
 	}
 
-	
+	geom->dev->boot_dirty = 1;
+
 	f = (struct ext2_fs *) fs->type_specific;
 
-	geom->disk->dev->boot_dirty = 1;
-	
 /* ensure that the geometry contains the new and old geometry */
 	if (old_length > geom->length) {
-		if (!ext2_resize_fs(f, geom->length >> (f->logsize - 9)))
+		if (!ext2_resize_fs(f, geom->length >> (f->logsize - 9),
+				    timer))
 			goto error;
 
 		fs->geom->length = geom->length;
@@ -293,124 +223,130 @@ _ext2_resize (PedFileSystem* fs, const PedGeometry* geom)
 		fs->geom->length = geom->length;
 		fs->geom->end = fs->geom->start + geom->length - 1;
 
-		if (!ext2_resize_fs(f, geom->length >> (f->logsize - 9)))
+		if (!ext2_resize_fs(f, geom->length >> (f->logsize - 9),
+				    timer))
 			goto error;
 	}
 	return 1;
 
 error:
-	fs->geom->length = old_length;
-	fs->geom->end = fs->geom->start + old_length - 1;
 	return 0;
 }
 
+static PedConstraint*
+_ext2_get_create_constraint (const PedDevice* dev)
+{
+	PedGeometry	full_dev;
+
+	if (!ped_geometry_init (&full_dev, dev, 0, dev->length - 1))
+		return NULL;
+
+	return ped_constraint_new (
+			ped_alignment_any, ped_alignment_any,
+			&full_dev, &full_dev,
+			64, dev->length);
+}
+
 static PedConstraint*
 _ext2_get_resize_constraint (const PedFileSystem* fs)
 {
 	struct ext2_fs* f = (struct ext2_fs *) fs->type_specific;
-	PedDisk*	disk = fs->geom->disk;
+	PedDevice*	dev = fs->geom->dev;
 	PedAlignment	start_align;
 	PedGeometry	start_sector;
-	PedGeometry	full_disk;
+	PedGeometry	full_dev;
 	PedSector	min_size;
 
 	if (!ped_alignment_init (&start_align, fs->geom->start, 0))
 		return NULL;
-	if (!ped_geometry_init (&full_disk, disk, 0, disk->dev->length - 1))
+	if (!ped_geometry_init (&full_dev, dev, 0, dev->length - 1))
 		return NULL;
-	if (!ped_geometry_init (&start_sector, disk, fs->geom->start, 1))
+	if (!ped_geometry_init (&start_sector, dev, fs->geom->start, 1))
 		return NULL;
 	min_size = (EXT2_SUPER_BLOCKS_COUNT(f->sb)
 		   	- EXT2_SUPER_FREE_BLOCKS_COUNT(f->sb))
 		   * (f->blocksize / PED_SECTOR_SIZE);
 
 	return ped_constraint_new (&start_align, ped_alignment_any,
-				   &start_sector, &full_disk, min_size);
+				   &start_sector, &full_dev, min_size,
+				   dev->length);
 }
+#endif /* !DISCOVER_ONLY */
 
-static int
-_ext2_set_system (const PedFileSystem* fs, PedPartition* part,
-		  const PedDiskType* disk_type)
+static PedFileSystemOps _ext2_ops = {
+	probe:		_ext2_probe,
+#ifndef DISCOVER_ONLY
+	clobber:	_ext2_clobber,
+	open:		_ext2_open,
+	create:         _ext2_create,
+	close:		_ext2_close,
+	check:          _ext2_check,
+	resize:		_ext2_resize,
+	copy:           NULL,
+	get_create_constraint:	_ext2_get_create_constraint,
+	get_copy_constraint:	NULL,
+	get_resize_constraint:	_ext2_get_resize_constraint
+#else /* !DISCOVER_ONLY */
+	clobber:	NULL,
+	open:		NULL,
+	create:         NULL,
+	close:		NULL,
+	check:          NULL,
+	resize:		NULL,
+	copy:           NULL,
+	get_create_constraint:	NULL,
+	get_copy_constraint:	NULL,
+	get_resize_constraint:	NULL
+#endif /* !DISCOVER_ONLY */
+};
+
+static PedFileSystemOps _ext3_ops = {
+	probe:		_ext3_probe,
+#ifndef DISCOVER_ONLY
+	clobber:	_ext2_clobber,
+	open:		_ext2_open,
+	create:         NULL,
+	close:		_ext2_close,
+	check:          _ext2_check,
+	resize:		_ext2_resize,
+	copy:           NULL,
+	get_create_constraint:	_ext2_get_create_constraint,
+	get_copy_constraint:	NULL,
+	get_resize_constraint:	_ext2_get_resize_constraint
+#else /* !DISCOVER_ONLY */
+	clobber:	NULL,
+	open:		NULL,
+	create:         NULL,
+	close:		NULL,
+	check:          NULL,
+	resize:		NULL,
+	copy:           NULL,
+	get_create_constraint:	NULL,
+	get_copy_constraint:	NULL,
+	get_resize_constraint:	NULL
+#endif /* !DISCOVER_ONLY */
+};
+
+static PedFileSystemType _ext2_type = {
+       next:		 NULL,
+       ops:		 &_ext2_ops,
+       name:		 "ext2"
+};
+
+static PedFileSystemType _ext3_type = {
+       next:		 NULL,
+       ops:		 &_ext3_ops,
+       name:		 "ext3"
+};
+
+void ped_file_system_ext2_init ()
 {
-	if (strcmp (disk_type->name, LOOP_NAME) == 0)
-		return 1;
-
-	if (strcmp (disk_type->name, DOS_NAME) == 0) {
-		DosPartitionData*	dos_data = part->disk_specific;
-
-		PED_ASSERT (dos_data != NULL, return 0);
-
-		if (dos_data->raid) {
-			dos_data->system = 0xfd;
-			return 1;
-		}
-		if (dos_data->lvm) {
-			dos_data->system = 0x8e;
-			return 1;
-		}
-
-		if (dos_data->hidden) {
-			ped_exception_throw (
-				PED_EXCEPTION_ERROR,
-			       	PED_EXCEPTION_FIX,
-				_("Ext2 partitions can't be hidden on msdos "
-				  "disk labels."));
-			dos_data->hidden = 0;
-		}
-
-		dos_data->system = 0x83;
-		return 1;
-	}
-
-	if (strcmp (disk_type->name, PC98_NAME) == 0) {
-		PC98PartitionData*	pc98_data = part->disk_specific;
-
-		PED_ASSERT (pc98_data != NULL, return 0);
-
-		pc98_data->boot = 1;
-		pc98_data->system = 0xa0e2;
-		return 1;
-	}
-
-	if (strcmp (disk_type->name, MAC_NAME) == 0) {
-		MacPartitionData*	mac_data = part->disk_specific;
-
-		PED_ASSERT (mac_data != NULL, return 0);
-
-		if (mac_data->is_boot)
-			strcpy (mac_data->system_name, "Apple_Bootstrap");
-		else
-			strcpy (mac_data->system_name, "Apple_UNIX_SVR2");
-		mac_data->status = 0x33;
-		return 1;
-	}
-
-	if (strcmp (disk_type->name, BSD_NAME) == 0) {
-		BSDPartitionData* bsd_data = part->disk_specific;
-		bsd_data->type = 0x8;
-		return 1;
-	}
-
-	if (strcmp (disk_type->name, GPT_NAME) == 0) {
-		GPTPartitionData *gpt_part_data = part->disk_specific;
-		PED_ASSERT(gpt_part_data      != NULL, return 0);	
-		PED_ASSERT(gpt_part_data->pte != NULL, return 0);
-		gpt_part_data->pte->PartitionTypeGuid =
-			PARTITION_BASIC_DATA_GUID;
-		return 1;
-	}
-
-	if (strcmp (disk_type->name, SUN_NAME) == 0) {
-		SunPartitionData* sun_data = part->disk_specific;
-
-		if (sun_data->is_lvm) {
-			sun_data->type = 0x8e;
-			return 1;
-		}
-
-		sun_data->type = 0x83;
-		return 1;
-	}
-
-	return 0;
+	ped_file_system_type_register (&_ext2_type);
+	ped_file_system_type_register (&_ext3_type);
+}
+
+void ped_file_system_ext2_done ()
+{
+	ped_file_system_type_unregister (&_ext2_type);
+	ped_file_system_type_unregister (&_ext3_type);
 }
