@@ -75,24 +75,27 @@ fat_boot_sector_read (FatBootSector* bs, const PedGeometry *geom)
 
 /*
     Don't trust the FAT12, FAT16 or FAT32 label string.
-*/
-static int
-probe_fat_type (PedFileSystem* fs)
+ */
+FatType
+fat_boot_sector_probe_type (const FatBootSector* bs, const PedGeometry* geom)
 {
-	FatSpecific*		fs_info = FAT_SPECIFIC (fs);
-	int			first_cluster_sector;
-	int			cluster_count;
+	PedSector	logical_sector_size;
+	PedSector	first_cluster_sector;
+	FatCluster	cluster_count;
 
-	if (!PED_LE16_TO_CPU (fs_info->boot_sector.dir_entries))
+	if (!PED_LE16_TO_CPU (bs->dir_entries))
 		return FAT_TYPE_FAT32;
 
+	logical_sector_size = PED_LE16_TO_CPU (bs->sector_size) / 512;
+
 	first_cluster_sector
-		= fs_info->fat_offset
-		  + 2 * PED_LE16_TO_CPU (fs_info->boot_sector.fat_length)
-		  + PED_LE16_TO_CPU (fs_info->boot_sector.dir_entries)
+		= PED_LE16_TO_CPU (bs->reserved) * logical_sector_size
+		  + 2 * PED_LE16_TO_CPU (bs->fat_length) * logical_sector_size
+		  + PED_LE16_TO_CPU (bs->dir_entries)
 			/ (512 / sizeof (FatDirEntry));
-	cluster_count = (fs_info->sector_count - first_cluster_sector)
-			/ fs_info->cluster_sectors;
+	cluster_count = (geom->length - first_cluster_sector)
+			/ PED_LE16_TO_CPU (bs->cluster_size)
+			/ logical_sector_size;
 	if (cluster_count > MAX_FAT12_CLUSTERS)
 		return FAT_TYPE_FAT16;
 	else
@@ -136,27 +139,6 @@ fat_boot_sector_analyse (FatBootSector* bs, PedFileSystem* fs)
 		fs_info->sector_count = PED_LE32_TO_CPU (bs->sector_count)
 						* fs_info->logical_sector_size;
 
-	if (fs_info->sector_count != fs->geom->length) {
-		ex_status = ped_exception_throw (PED_EXCEPTION_WARNING,
-			PED_EXCEPTION_IGNORE_CANCEL,
-			_("Partition size (%ld sectors) and filesystem "
-			  "size (%ld sectors) do not match."),
-			(long) fs->geom->length,
-			(long) fs_info->sector_count);
-
-		switch (ex_status) {
-			case PED_EXCEPTION_CANCEL:
-				return 0;
-
-			case PED_EXCEPTION_UNHANDLED:
-				ped_exception_catch ();
-				if (fs->geom->length < fs_info->sector_count)
-					return 0;
-			case PED_EXCEPTION_IGNORE:
-		                ; // just workaround for gcc 3.0
-		}
-	}
-
 	fs_info->fat_table_count = bs->fats;
 	fs_info->root_dir_entry_count = PED_LE16_TO_CPU (bs->dir_entries);
 	fs_info->fat_offset = PED_LE16_TO_CPU (bs->reserved)
@@ -184,7 +166,7 @@ fat_boot_sector_analyse (FatBootSector* bs, PedFileSystem* fs)
 		return 0;
 	}
 
-	fs_info->fat_type = probe_fat_type (fs);
+	fs_info->fat_type = fat_boot_sector_probe_type (bs, fs->geom);
 	if (fs_info->fat_type == FAT_TYPE_FAT12) {
 		ped_exception_throw (
 			PED_EXCEPTION_NO_FEATURE,
@@ -211,14 +193,14 @@ fat_boot_sector_analyse (FatBootSector* bs, PedFileSystem* fs)
 	if (fs_info->fat_type == FAT_TYPE_FAT32) {
 		fs_info->fat_sectors = PED_LE32_TO_CPU (bs->u.fat32.fat_length)
 					* fs_info->logical_sector_size;
+		fs_info->serial_number
+			= PED_LE32_TO_CPU (bs->u.fat32.serial_number);
 		fs_info->info_sector_offset
 		    = PED_LE16_TO_CPU (fs_info->boot_sector.u.fat32.info_sector)
 			  * fs_info->logical_sector_size;
 		fs_info->boot_sector_backup_offset
 		  = PED_LE16_TO_CPU (fs_info->boot_sector.u.fat32.backup_sector)
 			  * fs_info->logical_sector_size;
-		fs_info->serial_number
-			= PED_LE32_TO_CPU (bs->u.fat32.serial_number);
 		fs_info->root_cluster
 			= PED_LE32_TO_CPU (bs->u.fat32.root_dir_cluster);
 		fs_info->root_dir_offset = 0;
@@ -234,27 +216,16 @@ fat_boot_sector_analyse (FatBootSector* bs, PedFileSystem* fs)
 
 	fat_entry_size = fat_table_entry_size (fs_info->fat_type);
 	if (fs_info->cluster_count + 2
-			> fs_info->fat_sectors * 512 / fat_entry_size) {
-		ped_exception_throw (
-			PED_EXCEPTION_ERROR,
-			PED_EXCEPTION_CANCEL,
-			_("The FATs aren't big enough to describe all "
-			  "clusters!  Each FAT is %d sectors.  There are %d "
-			  "clusters, which would require each FAT to be %d "
-			  "sectors.  This is REALLY weird.  You might want to "
-			  "write us an email: bug-parted@gnu.org"),
-			(int) fs_info->fat_sectors,
-			(int) fs_info->cluster_count,
-			(int) div_round_up ((fs_info->cluster_count + 2)
-						* fat_entry_size, 512));
-		return 0;
-	}
+			> fs_info->fat_sectors * 512 / fat_entry_size)
+		fs_info->cluster_count
+			= fs_info->fat_sectors * 512 / fat_entry_size - 2;
 
 	fs_info->dir_entries_per_cluster
 		= fs_info->cluster_size / sizeof (FatDirEntry);
 	return 1;
 }
 
+#ifndef DISCOVER_ONLY
 int
 fat_boot_sector_set_boot_code (FatBootSector* bs)
 {
@@ -299,8 +270,8 @@ fat_boot_sector_generate (FatBootSector* bs, const PedFileSystem* fs)
 
 	bs->media = 0xf8;
 
-	sectors = fs->geom->disk->dev->sectors;
-	heads = fs->geom->disk->dev->heads;
+	sectors = fs->geom->dev->sectors;
+	heads = fs->geom->dev->heads;
 	bs->secs_track = PED_CPU_TO_LE16 (sectors);
 	bs->heads = PED_CPU_TO_LE16 (heads);
 	bs->hidden = PED_CPU_TO_LE32 (fs->geom->start);
@@ -420,4 +391,5 @@ fat_info_sector_write (const FatInfoSector* is, PedFileSystem *fs)
 		return 0;
 	return ped_geometry_sync (fs->geom);
 }
+#endif /* !DISCOVER_ONLY */
 
