@@ -21,6 +21,8 @@
 
 #include "fat.h"
 
+#ifndef DISCOVER_ONLY
+
 static int
 needs_duplicating (const FatOpContext* ctx, FatFragment frag)
 {
@@ -323,6 +325,8 @@ write_fragments (FatOpContext* ctx)
 		if (ctx->buffer_map [i] == -1)
 			continue;
 
+		ctx->frags_duped++;
+
 		new_cluster = fat_table_alloc_cluster (new_fs_info->fat);
 		if (!new_cluster)
 			return 0;
@@ -360,7 +364,7 @@ write_fragments (FatOpContext* ctx)
 
 /*  default all fragments to unmoved
  */
-void
+static void
 init_remap (FatOpContext* ctx)
 {
 	FatSpecific*		old_fs_info = FAT_SPECIFIC (ctx->old_fs);
@@ -370,22 +374,52 @@ init_remap (FatOpContext* ctx)
 		ctx->remap[i] = fat_op_context_map_static_fragment (ctx, i);
 }
 
+static FatFragment
+count_frags_to_dup (FatOpContext* ctx)
+{
+	FatSpecific*	fs_info = FAT_SPECIFIC (ctx->old_fs);
+	FatFragment	i;
+	FatFragment	total;
+
+	total = 0;
+
+	for (i = 0; i < fs_info->frag_count; i++) {
+		if (needs_duplicating (ctx, i))
+			total++;
+	}
+
+	return total;
+}
+
 /*  duplicates unreachable file clusters, and all directory clusters
  */
 int
-fat_duplicate_clusters (FatOpContext* ctx)
+fat_duplicate_clusters (FatOpContext* ctx, PedTimer* timer)
 {
+	FatSpecific*	old_fs_info = FAT_SPECIFIC (ctx->old_fs);
+	FatFragment	total_frags_to_dup;
+
 	init_remap (ctx);
+	total_frags_to_dup = count_frags_to_dup (ctx);
+
+	ped_timer_reset (timer);
+	ped_timer_set_state_name (timer, "moving data");
 
 	ctx->buffer_offset = 0;
-	while (1) {
-		if (!search_next_fragment (ctx))
-			return 1;
+	ctx->frags_duped = 0;
+	while (search_next_fragment (ctx)) {
+		ped_timer_update (
+			timer, 1.0 * ctx->frags_duped / total_frags_to_dup);
+
 		if (!fetch_fragments (ctx))
 			return 0;
 		if (!write_fragments (ctx))
 			return 0;
 		ctx->buffer_offset += ctx->buffer_frags;
 	}
+
+	ped_timer_update (timer, 1.0);
+	return 1;
 }
 
+#endif /* !DISCOVER_ONLY */
