@@ -24,17 +24,15 @@
 
 #include <parted/parted.h>
 #include <parted/endian.h>
+#include <parted/debug.h>
 
-#include <libintl.h>
 #if ENABLE_NLS
+#  include <libintl.h>
 #  define _(String) dgettext (PACKAGE, String)
 #else
 #  define _(String) (String)
 #endif /* ENABLE_NLS */
 
-// #define DEBUG
-// #define VERBOSE
-
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -42,8 +40,8 @@
 
 #define BUFFER_SIZE  1024	/* buffer size in sectors (512 bytes) */
 
-typedef __u32			FatCluster;
-typedef __s32			FatFragment;
+typedef uint32_t		FatCluster;
+typedef int32_t			FatFragment;
 
 enum _FatType {
 	FAT_TYPE_FAT12,
@@ -66,19 +64,19 @@ typedef struct _FatDirEntry	FatDirEntry;
 #include "clstdup.h"
 
 struct _FatDirEntry {
-	__u8		name[8];
-	__u8		extension[3];
-	__u8		attributes;
-	__u8		is_upper_case_name;
-	__u8		creation_time_low;      /* milliseconds */
-	__u16		creation_time_high;
-	__u16		creation_date;
-	__u16		access_date;
-	__u16		first_cluster_high;     /* for FAT32 */
-	__u16		time;
-	__u16		date;
-	__u16		first_cluster;
-	__u32		length;
+	uint8_t		name[8];
+	uint8_t		extension[3];
+	uint8_t		attributes;
+	uint8_t		is_upper_case_name;
+	uint8_t		creation_time_low;      /* milliseconds */
+	uint16_t	creation_time_high;
+	uint16_t	creation_date;
+	uint16_t	access_date;
+	uint16_t	first_cluster_high;     /* for FAT32 */
+	uint16_t	time;
+	uint16_t	date;
+	uint16_t	first_cluster;
+	uint32_t	length;
 } __attribute__ ((packed));
 
 struct _FatSpecific {
@@ -97,7 +95,7 @@ struct _FatSpecific {
 	int		fat_table_count;
 	PedSector	fat_sectors;
 
-	__u32		serial_number;
+	uint32_t	serial_number;
 
 	PedSector	info_sector_offset;     /* FAT32 only */
 	PedSector	fat_offset;
@@ -129,7 +127,7 @@ struct _FatSpecific {
 
 #define DELETED_FLAG		0xe5
 
-#define READ_ONLY_ATTR		0x01
+#define READONLY_ATTR		0x01
 #define HIDDEN_ATTR		0x02
 #define SYSTEM_ATTR		0x04
 #define VOLUME_LABEL_ATTR	0x08
@@ -142,17 +140,9 @@ struct _FatSpecific {
 
 #define FAT_ROOT_DIR_ENTRY_COUNT	512
 
-extern int fat_probe (const PedGeometry* geom);
-extern int fat_clobber (PedGeometry* geom);
-extern PedFileSystem* fat_open (const PedGeometry* geom);
-extern PedFileSystem* fat_create (PedGeometry* geom);
-extern int fat_close (PedFileSystem* fs);
-extern int fat_resize (PedFileSystem* fs, const PedGeometry* geom);
-extern int fat_copy (const PedFileSystem* fs, PedGeometry* geom);
-extern int fat_check (PedFileSystem* fs);
-extern PedConstraint* fat_get_resize_constraint (const PedFileSystem* fs);
-extern int fat_set_system (const PedFileSystem* fs, PedPartition* part,
-			    const PedDiskType* disk_type);
+extern PedFileSystemType fat16_type;
+extern PedFileSystemType fat32_type;
+
 extern void fat_print (const PedFileSystem* fs);
 
 extern PedFileSystem* fat_alloc (const PedGeometry* geom);
@@ -160,17 +150,8 @@ extern void fat_free (PedFileSystem* fs);
 extern int fat_alloc_buffers (PedFileSystem* fs);
 extern void fat_free_buffers (PedFileSystem* fs);
 
+extern int fat_resize (PedFileSystem* fs, PedGeometry* geom, PedTimer* timer);
+
 extern int fat_set_frag_sectors (PedFileSystem* fs, PedSector frag_sectors);
 
-inline static PedSector
-div_round_up (PedSector a, PedSector b)
-{
-	PedSector	result;
-
-	result = a / b;
-	if (a % b) result++;
-
-	return result;
-}
-
 #endif /* FAT_H_INCLUDED */
