@@ -22,6 +22,8 @@
 #include <parted/endian.h>
 #include "fat.h"
 
+#ifndef DISCOVER_ONLY
+
 FatTable*
 fat_table_new (FatType fat_type, FatCluster size)
 {
@@ -34,7 +36,7 @@ fat_table_new (FatType fat_type, FatCluster size)
 	ft->cluster_count = ft->free_cluster_count = size - 2;
 
 /* ensure there's some free room on the end, to finish off the sector */
-	ft->size = div_round_up (size * entry_size, 512) * 512 / entry_size;
+	ft->size = ped_div_round_up (size * entry_size, 512) * 512 / entry_size;
 	ft->fat_type = fat_type;
 	ft->raw_size = ft->size * entry_size;
 
@@ -444,6 +446,8 @@ fat_table_set_avail (FatTable* ft, FatCluster cluster)
 	return fat_table_set (ft, cluster, 0);
 }
 
+#endif /* !DISCOVER_ONLY */
+
 int
 fat_table_entry_size (FatType fat_type)
 {
